@@ -2,7 +2,7 @@
 
 """theses tests are much more to test that config, option description, vs...
 **it's there** and answers via attribute access"""
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
@@ -165,6 +165,9 @@ def test_information_config():
     assert config.impl_get_information('info') == string
     raises(ValueError, "config.impl_get_information('noinfo')")
     assert config.impl_get_information('noinfo', 'default') == 'default'
+    config.impl_del_information('info')
+    raises(ValueError, "config.impl_get_information('info')")
+    raises(ValueError, "config.impl_del_information('noinfo')")
 
 
 def test_config_impl_get_path_by_opt():
@@ -192,8 +195,12 @@ def test_config_impl_get_opt_by_path():
     config = Config(descr)
     dummy = config.unwrap_from_path('gc.dummy')
     boo = config.unwrap_from_path('bool')
-    assert config.cfgimpl_get_description().impl_get_opt_by_path('bool') == boo
-    assert config.cfgimpl_get_description().impl_get_opt_by_path('gc.dummy') == dummy
+    if 'id' in dir(boo):
+        assert config.cfgimpl_get_description().impl_get_opt_by_path('bool').id == boo.id
+        assert config.cfgimpl_get_description().impl_get_opt_by_path('gc.dummy').id == dummy.id
+    else:
+        assert config.cfgimpl_get_description().impl_get_opt_by_path('bool') == boo
+        assert config.cfgimpl_get_description().impl_get_opt_by_path('gc.dummy') == dummy
     raises(AttributeError, "config.cfgimpl_get_description().impl_get_opt_by_path('gc.unknown')")
 
 
@@ -220,7 +227,8 @@ def test_get_modified_values():
     g3 = UnicodeOption('g3', '', u'héhé')
     g4 = BoolOption('g4', '', True)
     g5 = StrOption('g5', '')
-    d1 = OptionDescription('od', '', [g1, g2, g3, g4, g5])
+    g6 = StrOption('g6', '', multi=True)
+    d1 = OptionDescription('od', '', [g1, g2, g3, g4, g5, g6])
     root = OptionDescription('root', '', [d1])
     config = Config(root)
     assert config.cfgimpl_get_values().get_modified_values() == {}
@@ -228,6 +236,16 @@ def test_get_modified_values():
     assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes')}
     config.od.g4 = True
     assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes'), 'od.g4': ('user', True)}
+    del config.od.g4
+    assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes')}
+    config.od.g6.append()
+    assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes'), 'od.g6': ('user', (None,))}
+    config.od.g6.pop(0)
+    assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes'), 'od.g6': ('user', tuple())}
+    config.od.g6.append('3')
+    assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes'), 'od.g6': ('user', ('3',))}
+    del config.od.g6[0]
+    assert config.cfgimpl_get_values().get_modified_values() == {'od.g5': ('user', 'yes'), 'od.g6': ('user', tuple())}
 
 
 def test_has_value():
