@@ -1,4 +1,4 @@
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 import warnings, sys
@@ -54,7 +54,7 @@ def test_domainname_upper():
         if sys.version_info[0] >= 3:  # pragma: optional cover
             assert msg in str(err)
         else:
-            assert msg in unicode(err)
+            assert msg in str(err)
         has_error = True
     assert has_error is True
     has_error = False
@@ -64,7 +64,7 @@ def test_domainname_upper():
         if sys.version_info[0] >= 3:  # pragma: optional cover
             assert msg in str(err)
         else:
-            assert msg in unicode(err)
+            assert msg in str(err)
         has_error = True
     assert has_error is True
 
@@ -143,10 +143,12 @@ def test_email():
     od = OptionDescription('a', '', [e])
     c = Config(od)
     c.read_write()
-    c.e = 'root@foo.com'
-    raises(ValueError, "c.e = 'root'")
-    raises(ValueError, "c.e = 'root@domain'")
-    raises(ValueError, "c.e = 'root[]@domain'")
+    c.e = u'foo-bar.baz@example.com'
+    c.e = u'root@foo.com'
+    c.e = u'root@domain'
+    raises(ValueError, "c.e = 1")
+    raises(ValueError, "c.e = u'root'")
+    raises(ValueError, "c.e = u'root[]@domain'")
 
 
 def test_url():
@@ -157,6 +159,7 @@ def test_url():
     c.u = 'http://foo.com'
     c.u = 'https://foo.com'
     c.u = 'https://foo.com/'
+    raises(ValueError, "c.u = 1")
     raises(ValueError, "c.u = 'ftp://foo.com'")
     raises(ValueError, "c.u = 'foo.com'")
     raises(ValueError, "c.u = ':/foo.com'")
