@@ -1,11 +1,11 @@
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 import warnings
 from py.test import raises
 from tiramisu.config import Config
 from tiramisu.option import IPOption, NetworkOption, NetmaskOption, \
-    PortOption, OptionDescription
+    PortOption, BroadcastOption, OptionDescription
 from tiramisu.error import ValueWarning
 
 
@@ -26,6 +26,7 @@ def test_ip():
     raises(ValueError, "c.b = '88.88.88.88'")
     c.b = '0.0.0.0'
     raises(ValueError, "c.b = '255.255.255.0'")
+    raises(ValueError, "c.a = '333.0.1.20'")
 
     raises(ValueError, "IPOption('a', 'ip', default='192.000.023.01')")
     with warnings.catch_warnings(record=True) as w:
@@ -64,7 +65,11 @@ def test_network():
     c.a = '192.168.1.0'
     c.a = '88.88.88.88'
     c.a = '0.0.0.0'
+    raises(ValueError, "c.a = 1")
+    raises(ValueError, "c.a = '1.1.1.1.1'")
     raises(ValueError, "c.a = '255.255.255.0'")
+    raises(ValueError, "c.a = '192.168.001.0'")
+    raises(ValueError, "c.a = '333.168.1.1'")
     with warnings.catch_warnings(record=True) as w:
         c.b = '255.255.255.0'
     assert len(w) == 1
@@ -78,9 +83,25 @@ def test_netmask():
     a = NetmaskOption('a', '')
     od = OptionDescription('od', '', [a])
     c = Config(od)
+    raises(ValueError, "c.a = '192.168.1.1.1'")
     raises(ValueError, "c.a = '192.168.1.1'")
     raises(ValueError, "c.a = '192.168.1.0'")
     raises(ValueError, "c.a = '88.88.88.88'")
+    raises(ValueError, "c.a = '255.255.255.000'")
+    raises(ValueError, "c.a = 2")
+    c.a = '0.0.0.0'
+    c.a = '255.255.255.0'
+
+
+def test_broadcast():
+    a = BroadcastOption('a', '')
+    od = OptionDescription('od', '', [a])
+    c = Config(od)
+    raises(ValueError, "c.a = '192.168.1.255.1'")
+    raises(ValueError, "c.a = '192.168.001.255'")
+    raises(ValueError, "c.a = '192.168.0.300'")
+    raises(ValueError, "c.a = 1")
+    raises(ValueError, "c.a = 2")
     c.a = '0.0.0.0'
     c.a = '255.255.255.0'
 
