@@ -1,5 +1,5 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from tiramisu.config import Config, GroupConfig, MetaConfig
@@ -87,6 +87,8 @@ def test_deref_option_cache():
 
 
 def test_deref_optiondescription_cache():
+    if not IS_DEREFABLE:
+        return
     b = BoolOption('b', '')
     o = OptionDescription('od', '', [b])
     o.impl_build_cache_option()
@@ -113,6 +115,8 @@ def test_deref_option_config():
 
 
 def test_deref_optiondescription_config():
+    if not IS_DEREFABLE:
+        return
     b = BoolOption('b', '')
     o = OptionDescription('od', '', [b])
     c = Config(o)
@@ -126,6 +130,8 @@ def test_deref_optiondescription_config():
 
 
 def test_deref_groupconfig():
+    if not IS_DEREFABLE:
+        return
     i1 = IntOption('i1', '')
     od1 = OptionDescription('od1', '', [i1])
     od2 = OptionDescription('od2', '', [od1])
@@ -140,6 +146,8 @@ def test_deref_groupconfig():
 
 
 def test_deref_metaconfig():
+    if not IS_DEREFABLE:
+        return
     i1 = IntOption('i1', '')
     od1 = OptionDescription('od1', '', [i1])
     od2 = OptionDescription('od2', '', [od1])
@@ -154,6 +162,8 @@ def test_deref_metaconfig():
 
 
 def test_deref_submulti():
+    if not IS_DEREFABLE:
+        return
     multi = StrOption('multi', '', multi=submulti)
     od = OptionDescription('od', '', [multi])
     cfg = Config(od)
