@@ -1,5 +1,5 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from tiramisu.setting import groups, owners
@@ -107,8 +107,8 @@ def test_getdoc_dyndescription():
     assert dodval2.impl_getname() == 'dodval2'
     assert stval1.impl_getdoc() == 'doc1'
     assert stval2.impl_getdoc() == 'doc1'
-    assert dodval1.impl_getdoc() == 'doc2'
-    assert dodval2.impl_getdoc() == 'doc2'
+    assert dodval1.impl_getdoc() == 'doc2val1'
+    assert dodval2.impl_getdoc() == 'doc2val2'
 
 
 def test_getpaths_dyndescription():
@@ -561,6 +561,66 @@ def test_requires_dyndescription():
     assert props == ['disabled']
 
 
+def test_requires_dyndescription_boolean():
+    boolean1 = BoolOption('boolean1', '', True)
+    boolean = BoolOption('boolean', '', True, requires=[{'option': boolean1,
+                                                         'expected': False,
+                                                         'action': 'disabled'}])
+    st = StrOption('st', '', requires=[{'option': boolean, 'expected': False,
+                                        'action': 'disabled'}])
+    dod = DynOptionDescription('dod', '', [st], callback=return_list)
+    od = OptionDescription('od', '', [dod])
+    od2 = OptionDescription('od', '', [od, boolean1, boolean])
+    cfg = Config(od2)
+    cfg.read_write()
+    assert cfg.make_dict() == {'boolean1': True,
+                               'boolean': True,
+                               'od.dodval1.stval1': None,
+                               'od.dodval2.stval2': None}
+    #
+    cfg.boolean = False
+    assert cfg.make_dict() == {'boolean1': True,
+                               'boolean': False}
+    #
+    cfg.boolean = True
+    assert cfg.make_dict() == {'boolean1': True,
+                               'boolean': True,
+                               'od.dodval1.stval1': None,
+                               'od.dodval2.stval2': None}
+    #
+    cfg.boolean1 = False
+    assert cfg.make_dict() == {'boolean1': False}
+
+
+def test_requires_dyndescription_in_dyn():
+    boolean = BoolOption('boolean', '', True)
+    st = StrOption('st', '', requires=[{'option': boolean, 'expected': False,
+                                        'action': 'disabled'}])
+    dod = DynOptionDescription('dod', '', [boolean, st], callback=return_list)
+    od = OptionDescription('od', '', [dod])
+    od2 = OptionDescription('od', '', [od])
+    cfg = Config(od2)
+    cfg.read_write()
+
+    assert cfg.od.dodval1.stval1 is None
+    assert cfg.od.dodval2.stval2 is None
+    #
+    cfg.od.dodval1.booleanval1 = False
+
+    props = []
+    try:
+        cfg.od.dodval1.stval1
+    except PropertiesOptionError as err:
+        props = err.proptype
+    assert props == ['disabled']
+    props = []
+    cfg.od.dodval2.stval2
+    #
+    cfg.od.dodval1.booleanval1 = True
+    assert cfg.od.dodval1.stval1 is None
+    assert cfg.od.dodval2.stval2 is None
+
+
 def test_requires_dyndescription2():
     boolean = BoolOption('boolean', '', True)
     st = StrOption('st', '')
@@ -646,6 +706,7 @@ def test_find_dyndescription_context():
     assert isinstance(cfg.find_first(byname='stval1', type_='option'), DynSymLinkOption)
     assert cfg.find(bytype=StrOption, type_='path') == ['od.dodval1.stval1', 'od.dodval2.stval2', 'od.val1']
     opts = cfg.find(byvalue='yes')
+    raises(AttributeError, "cfg.find(byname='strnotexists')")
     assert len(opts) == 1
     assert isinstance(opts[0], DynSymLinkOption)
     assert opts[0].impl_getname() == 'stval1'
@@ -990,6 +1051,284 @@ def test_masterslaves_dyndescription():
     assert cfg.getowner(st2val2) == owners.default
 
 
+def test_masterslaves_dyndescription_param():
+    val1 = StrOption('val1', '', ['val1', 'val2'], multi=True)
+    odval = OptionDescription('odval', '', [val1])
+    st1 = StrOption('st1', "", multi=True)
+    st2 = StrOption('st2', "", multi=True)
+    stm = OptionDescription('st1', '', [st1, st2])
+    stm.impl_set_group_type(groups.master)
+    st = DynOptionDescription('st', '', [stm], callback=return_list, callback_params={'': ((val1, False),)})
+    od = OptionDescription('od', '', [st, odval])
+    od2 = OptionDescription('od', '', [od])
+    cfg = Config(od2)
+    owner = cfg.cfgimpl_get_settings().getowner()
+    st1val1 = cfg.unwrap_from_path('od.stval1.st1val1.st1val1')
+    st2val1 = cfg.unwrap_from_path('od.stval1.st1val1.st2val1')
+    st1val2 = cfg.unwrap_from_path('od.stval2.st1val2.st1val2')
+    st2val2 = cfg.unwrap_from_path('od.stval2.st1val2.st2val2')
+    assert cfg.make_dict() == {'od.stval1.st1val1.st2val1': [], 'od.stval2.st1val2.st2val2': [], 'od.stval2.st1val2.st1val2': [], 'od.stval1.st1val1.st1val1': [], 'od.odval.val1': ['val1', 'val2']}
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owners.default
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1.append('yes')
+    assert cfg.make_dict() == {'od.stval1.st1val1.st2val1': [None], 'od.stval2.st1val2.st2val2': [], 'od.stval2.st1val2.st1val2': [], 'od.stval1.st1val1.st1val1': ['yes'], 'od.odval.val1': ['val1', 'val2']}
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == [None]
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == [None]
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st2val1 = ['no']
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == ['no']
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1, 0) == owner
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1.pop(0)
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+#    assert cfg.getowner(st2val1) == owner
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    cfg.od.stval1.st1val1.st2val1 = ['yes']
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st2val1, 0) == owner
+    del(cfg.od.stval1.st1val1.st2val1)
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+#    assert cfg.getowner(st2val1) == owners.default
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    cfg.od.stval1.st1val1.st2val1 = ['yes']
+    del(cfg.od.stval1.st1val1.st1val1)
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owners.default
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+
+
+def test_masterslaves_dyndescription_param_master():
+    val1 = StrOption('val1', "", multi=True)
+    val2 = StrOption('val2', "", multi=True)
+    odval = OptionDescription('val1', '', [val1, val2])
+    odval.impl_set_group_type(groups.master)
+    st1 = StrOption('st1', "", multi=True)
+    st2 = StrOption('st2', "", multi=True)
+    stm = OptionDescription('st1', '', [st1, st2])
+    stm.impl_set_group_type(groups.master)
+    st = DynOptionDescription('st', '', [stm], callback=return_list, callback_params={'': ((val1, False),)})
+    od = OptionDescription('od', '', [st, odval])
+    od2 = OptionDescription('od', '', [od])
+    cfg = Config(od2)
+    owner = cfg.cfgimpl_get_settings().getowner()
+    cfg.od.val1.val1 = ['val1', 'val2']
+    cfg.od.val1.val2 = ['val1', 'val2']
+    st1val1 = cfg.unwrap_from_path('od.stval1.st1val1.st1val1')
+    st2val1 = cfg.unwrap_from_path('od.stval1.st1val1.st2val1')
+    st1val2 = cfg.unwrap_from_path('od.stval2.st1val2.st1val2')
+    st2val2 = cfg.unwrap_from_path('od.stval2.st1val2.st2val2')
+    assert cfg.make_dict() == {'od.stval1.st1val1.st2val1': [], 'od.stval2.st1val2.st2val2': [], 'od.stval2.st1val2.st1val2': [], 'od.stval1.st1val1.st1val1': [], 'od.val1.val1': ['val1', 'val2'], 'od.val1.val2': ['val1', 'val2']}
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owners.default
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1.append('yes')
+    assert cfg.make_dict() == {'od.stval1.st1val1.st2val1': [None], 'od.stval2.st1val2.st2val2': [], 'od.stval2.st1val2.st1val2': [], 'od.stval1.st1val1.st1val1': ['yes'], 'od.val1.val1': ['val1', 'val2'], 'od.val1.val2': ['val1', 'val2']}
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == [None]
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == [None]
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st2val1 = ['no']
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == ['no']
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1, 0) == owner
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1.pop(0)
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+#    assert cfg.getowner(st2val1) == owner
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    cfg.od.stval1.st1val1.st2val1 = ['yes']
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st2val1, 0) == owner
+    del(cfg.od.stval1.st1val1.st2val1)
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+#    assert cfg.getowner(st2val1) == owners.default
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    cfg.od.stval1.st1val1.st2val1 = ['yes']
+    del(cfg.od.stval1.st1val1.st1val1)
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owners.default
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+
+
+def test_masterslaves_dyndescription_param_slave():
+    val1 = StrOption('val1', "", multi=True)
+    val2 = StrOption('val2', "", multi=True)
+    odval = OptionDescription('val1', '', [val1, val2])
+    odval.impl_set_group_type(groups.master)
+    st1 = StrOption('st1', "", multi=True)
+    st2 = StrOption('st2', "", multi=True)
+    stm = OptionDescription('st1', '', [st1, st2])
+    stm.impl_set_group_type(groups.master)
+    st = DynOptionDescription('st', '', [stm], callback=return_list, callback_params={'': ((val2, False),)})
+    od = OptionDescription('od', '', [st, odval])
+    od2 = OptionDescription('od', '', [od])
+    cfg = Config(od2)
+    owner = cfg.cfgimpl_get_settings().getowner()
+    cfg.od.val1.val1 = ['val1', 'val2']
+    cfg.od.val1.val2 = ['val1', 'val2']
+    st1val1 = cfg.unwrap_from_path('od.stval1.st1val1.st1val1')
+    st2val1 = cfg.unwrap_from_path('od.stval1.st1val1.st2val1')
+    st1val2 = cfg.unwrap_from_path('od.stval2.st1val2.st1val2')
+    st2val2 = cfg.unwrap_from_path('od.stval2.st1val2.st2val2')
+    assert cfg.make_dict() == {'od.stval1.st1val1.st2val1': [], 'od.stval2.st1val2.st2val2': [], 'od.stval2.st1val2.st1val2': [], 'od.stval1.st1val1.st1val1': [], 'od.val1.val1': ['val1', 'val2'], 'od.val1.val2': ['val1', 'val2']}
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owners.default
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1.append('yes')
+    assert cfg.make_dict() == {'od.stval1.st1val1.st2val1': [None], 'od.stval2.st1val2.st2val2': [], 'od.stval2.st1val2.st1val2': [], 'od.stval1.st1val1.st1val1': ['yes'], 'od.val1.val1': ['val1', 'val2'], 'od.val1.val2': ['val1', 'val2']}
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == [None]
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == [None]
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st2val1 = ['no']
+    assert cfg.od.stval1.st1val1.st1val1 == ['yes']
+    assert cfg.od.stval1.st1val1.st2val1 == ['no']
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1, 0) == owner
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1.pop(0)
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+#    assert cfg.getowner(st2val1) == owner
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    cfg.od.stval1.st1val1.st2val1 = ['yes']
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st2val1, 0) == owner
+    del(cfg.od.stval1.st1val1.st2val1)
+    assert cfg.getowner(st1val1) == owner
+    assert cfg.getowner(st1val2) == owners.default
+#    assert cfg.getowner(st2val1) == owners.default
+#    assert cfg.getowner(st2val2) == owners.default
+    #
+    cfg.od.stval1.st1val1.st1val1 = ['yes']
+    cfg.od.stval1.st1val1.st2val1 = ['yes']
+    del(cfg.od.stval1.st1val1.st1val1)
+    assert cfg.od.stval1.st1val1.st1val1 == []
+    assert cfg.od.stval1.st1val1.st2val1 == []
+    assert cfg.od.stval2.st1val2.st1val2 == []
+    assert cfg.od.stval2.st1val2.st2val2 == []
+    assert cfg.getowner(st1val1) == owners.default
+    assert cfg.getowner(st1val2) == owners.default
+    assert cfg.getowner(st2val1) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
+
+
 def test_masterslaves_default_multi_dyndescription():
     st1 = StrOption('st1', "", multi=True)
     st2 = StrOption('st2', "", multi=True, default_multi='no')
@@ -1063,7 +1402,7 @@ def test_masterslaves_submulti_dyndescription():
     assert cfg.getowner(st1val1) == owner
     assert cfg.getowner(st1val2) == owners.default
     assert cfg.getowner(st2val1, 0) == owner
-#    assert cfg.getowner(st2val2) == owners.default
+    assert cfg.getowner(st2val2) == owners.default
 
 
 def test_masterslaves_consistency_ip_dyndescription():
@@ -1093,6 +1432,22 @@ def test_masterslaves_consistency_ip_dyndescription():
     assert cfg.netval2.broadval2 == []
 
 
+def test_masterslaves_consistency_ip_dyndescription_propertyerror():
+    a = NetworkOption('net', '', multi=True)
+    b = NetmaskOption('mask', '', multi=True, properties=('mandatory',))
+    c = BroadcastOption('broad', '', multi=True)
+    b.impl_add_consistency('network_netmask', a)
+    c.impl_add_consistency('broadcast', a, b)
+    dod = DynOptionDescription('net', '', [a, b, c], callback=return_list)
+    dod.impl_set_group_type(groups.master)
+    od = OptionDescription('od', '', [dod])
+    cfg = Config(od)
+    cfg.read_write()
+    cfg.netval1.netval1 = ['192.168.1.0']
+    cfg.read_only()
+    raises(PropertiesOptionError, "cfg.netval1.netval1")
+
+
 def test_masterslaves_callback_dyndescription():
     st1 = StrOption('st1', "", multi=True)
     st2 = StrOption('st2', "", multi=True, callback=return_dynval, callback_params={'value': ((st1, False),)})
@@ -1293,10 +1648,10 @@ def test_state_config():
     except ValueError:
         cfg = Config(od2, session_id='29090938')
         cfg._impl_test = True
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
-    _diff_conf(cfg, q)
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
+        _diff_conf(cfg, q)
 
     try:
         delete_session('config', '29090938')
