@@ -1,6 +1,6 @@
 # coding: utf-8
 "frozen and hidden values"
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
@@ -170,15 +170,15 @@ def test_freeze_get_multi():
 def test_force_store_value():
     descr = make_description_freeze()
     conf = Config(descr)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'wantref': ('forced', False),
+    assert conf.cfgimpl_get_values().get_modified_values() == {'wantref': ('forced', False),
                                                                    'wantref2': ('forced', False),
                                                                    'wantref3': ('forced', (False,))}
     conf.wantref = True
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'wantref': ('user', True),
+    assert conf.cfgimpl_get_values().get_modified_values() == {'wantref': ('user', True),
                                                                    'wantref2': ('forced', False),
                                                                    'wantref3': ('forced', (False,))}
     del(conf.wantref)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'wantref': ('forced', False),
+    assert conf.cfgimpl_get_values().get_modified_values() == {'wantref': ('forced', False),
                                                                    'wantref2': ('forced', False),
                                                                    'wantref3': ('forced', (False,))}
 
@@ -206,28 +206,28 @@ def test_force_store_value_masterslaves():
     descr = OptionDescription("int", "", [b, c])
     descr.impl_set_group_type(groups.master)
     conf = Config(descr)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'int': ('forced', ())}
+    assert conf.cfgimpl_get_values().get_modified_values() == {'int': ('forced', ())}
 
 
 def test_force_store_value_callback():
     b = IntOption('int', 'Test int option', properties=('force_store_value',), callback=return_val)
     descr = OptionDescription("int", "", [b])
     conf = Config(descr)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'int': ('forced', 1)}
+    assert conf.cfgimpl_get_values().get_modified_values() == {'int': ('forced', 1)}
 
 
 def test_force_store_value_callback_params():
     b = IntOption('int', 'Test int option', properties=('force_store_value',), callback=return_val2, callback_params={'value': (2,)})
     descr = OptionDescription("int", "", [b])
     conf = Config(descr)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'int': ('forced', 2)}
+    assert conf.cfgimpl_get_values().get_modified_values() == {'int': ('forced', 2)}
 
 
 def test_force_store_value_callback_params_2():
     b = IntOption('int', 'Test int option', properties=('force_store_value',), callback=return_val3, callback_params={'': ((None,),), 'value': (2,)})
     descr = OptionDescription("int", "", [b])
     conf = Config(descr)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'int': ('forced', 2)}
+    assert conf.cfgimpl_get_values().get_modified_values() == {'int': ('forced', 2)}
 
 
 def test_force_store_value_callback_params_with_opt():
@@ -235,4 +235,4 @@ def test_force_store_value_callback_params_with_opt():
     b = IntOption('int', 'Test int option', properties=('force_store_value',), callback=return_val2, callback_params={'value': ((a, False),)})
     descr = OptionDescription("int", "", [a, b])
     conf = Config(descr)
-    assert conf.cfgimpl_get_values()._p_.get_modified_values() == {'int': ('forced', 2)}
+    assert conf.cfgimpl_get_values().get_modified_values() == {'int': ('forced', 2)}
