@@ -1,16 +1,20 @@
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
 
 from tiramisu.setting import groups, owners
 from tiramisu.config import Config, GroupConfig, MetaConfig
-from tiramisu.option import IntOption, StrOption, OptionDescription
+from tiramisu.option import IntOption, StrOption, NetworkOption, NetmaskOption, OptionDescription
 from tiramisu.error import ConfigError, ConflictError
 
 owners.addowner('meta')
 
 
+def raise_exception():
+    raise Exception('test')
+
+
 def make_description():
     i1 = IntOption('i1', '')
     i2 = IntOption('i2', '', default=1)
@@ -155,7 +159,6 @@ def test_meta_meta_set():
     meta2.cfgimpl_get_settings().setowner(owners.meta)
     conf1, conf2 = meta1.cfgimpl_get_children()
     meta2.set_value('od1.i1', 7, only_config=True)
-    #PropertiesOptionError
     meta2.set_value('od1.i6', 7, only_config=True)
     assert conf1.od1.i1 == conf2.od1.i1 == 7
     assert conf1.getowner(conf1.unwrap_from_path('od1.i1')) is conf2.getowner(conf2.unwrap_from_path('od1.i1')) is owners.user
@@ -500,3 +503,33 @@ def test_meta_force_default_and_dont_change():
     meta.read_write()
     meta.cfgimpl_get_settings().setowner(owners.meta)
     raises(ValueError, "meta.set_value('ip_admin_eth0', ['192.168.1.4'], force_default=True, force_dont_change_value=True)")
+
+
+def test_meta_properties_meta():
+    ip_admin_eth0 = NetworkOption('ip_admin_eth0', "ip", multi=True, default=['192.168.1.1'])
+    netmask_admin_eth0 = NetmaskOption('netmask_admin_eth0', "mask", multi=True, properties=('disabled',))
+    netmask_admin_eth0.impl_add_consistency('network_netmask', ip_admin_eth0)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    conf1 = Config(interface1, name='conf1')
+    conf1.read_write()
+    conf2 = Config(interface1, name='conf2')
+    conf2.read_write()
+    meta = MetaConfig([conf1, conf2])
+    meta.read_write()
+    assert conf1.make_dict() == {}
+
+
+def test_meta_exception_meta():
+    ip_admin_eth0 = NetworkOption('ip_admin_eth0', "ip", multi=True, default=['192.168.1.1'])
+    netmask_admin_eth0 = NetmaskOption('netmask_admin_eth0', "mask", multi=True, callback=raise_exception)
+    netmask_admin_eth0.impl_add_consistency('network_netmask', ip_admin_eth0)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    conf1 = Config(interface1, name='conf1')
+    conf1.read_write()
+    conf2 = Config(interface1, name='conf2')
+    conf2.read_write()
+    meta = MetaConfig([conf1, conf2])
+    meta.read_write()
+    raises(Exception, "conf1.make_dict()")
