@@ -1,4 +1,4 @@
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
@@ -53,6 +53,14 @@ def is_config(config, **kwargs):
         return 'no'
 
 
+def return_raise(*arg):
+    raise Exception('test')
+
+
+def return_valueerror(*arg):
+    raise ValueError('test')
+
+
 def make_description_duplicates():
     gcoption = ChoiceOption('name', 'GC name', ('ref', 'framework'), 'ref')
     ## dummy 1
@@ -345,6 +353,17 @@ def test_callback_list():
     raises(ValueError, "cfg.val1")
 
 
+def test_callback_list2():
+    val1 = StrOption('val1', "", callback=return_list)
+    val2 = StrOption('val2', "", callback=return_value, callback_params={'': ((val1, False),)})
+    maconfig = OptionDescription('rootconfig', '', [val1, val2])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    raises(ValueError, "cfg.val1")
+    #cfg.val2
+    raises(ValueError, "cfg.val2")
+
+
 def test_callback_multi():
     val1 = StrOption('val1', "", callback=return_val, multi=True)
     maconfig = OptionDescription('rootconfig', '', [val1])
@@ -436,6 +455,24 @@ def test_callback_master_and_slaves_master():
     assert cfg.val1.val2 == [None, None]
 
 
+def test_callback_slave():
+    val1 = StrOption('val1', "", multi=True)
+    val2 = StrOption('val2', "", multi=True, callback=return_value, callback_params={'': (['string', 'new'],)})
+    interface1 = OptionDescription('val1', '', [val1, val2])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    cfg.val1.val1 = ['val']
+    assert cfg.val1.val2 == ['string']
+    cfg.val1.val1 = ['val', 'val1']
+    assert cfg.val1.val2 == ['string', 'new']
+    cfg.val1.val1 = ['val', 'val1', 'val2']
+    assert cfg.val1.val2 == ['string', 'new', None]
+    cfg.val1.val1 = ['val', 'val1', 'val2', 'val3']
+    assert cfg.val1.val2 == ['string', 'new', None, None]
+
+
 def test_callback_master_and_slaves_master2():
     val1 = StrOption('val1', "", multi=True)
     val2 = StrOption('val2', "", multi=True, default_multi='val2')
@@ -452,6 +489,96 @@ def test_callback_master_and_slaves_master2():
     assert cfg.val1.val2 == ['val2']
 
 
+def test_callback_master_and_slaves_master_mandatory():
+    val = StrOption('val', "", default='val')
+    val1 = StrOption('val1', "", multi=True, callback=return_value2, callback_params={'': ((val, False),)}, properties=('mandatory',))
+    val3 = StrOption('val3', "", multi=True, callback=return_value2, callback_params={'': ((val1, False),)}, properties=('mandatory',))
+    val4 = StrOption('val4', "", multi=True, callback=return_value2, callback_params={'': ((val1, False),)}, properties=('mandatory',))
+    interface1 = OptionDescription('val1', '', [val1, val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [val, interface1])
+    cfg = Config(maconfig)
+    cfg.read_only()
+    assert cfg.val1.val3 == ['val']
+    assert cfg.val1.val4 == ['val']
+    assert cfg.val1.val1 == ['val']
+    cfg.read_write()
+    cfg.val1.val1.append('val3')
+    cfg.read_only()
+    assert cfg.val1.val1 == ['val', 'val3']
+    assert cfg.val1.val3[0] == 'val'
+    raises(PropertiesOptionError, 'cfg.val1.val3[1]')
+    raises(PropertiesOptionError, 'cfg.val1.val4[1]')
+
+
+def test_callback_master_and_slaves_master_mandatory2():
+    val = StrOption('val', "", default='val')
+    val_ = StrOption('val_', "", default='val_')
+    val1 = StrOption('val1', "", multi=True, callback=return_value2, callback_params={'': ((val, False),), 'val': ((val_, False),)}, properties=('mandatory',))
+    val3 = StrOption('val3', "", multi=True, callback=return_value2, callback_params={'': ((val1, False),), 'val': ((val_, False),)}, properties=('mandatory',))
+    val4 = StrOption('val4', "", multi=True, callback=return_value2, callback_params={'': ((val1, False),), 'val': ((val_, False),)}, properties=('mandatory',))
+    interface1 = OptionDescription('val1', '', [val1, val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [val, val_, interface1])
+    cfg = Config(maconfig)
+    cfg.read_only()
+    assert cfg.val1.val3 == ['val', 'val_']
+    assert cfg.val1.val4 == ['val', 'val_']
+    assert cfg.val1.val1 == ['val', 'val_']
+    cfg.read_write()
+    cfg.val1.val1.append('val3')
+    assert cfg.val1.val1 == ['val', 'val_', 'val3']
+    cfg.read_only()
+    assert cfg.val1.val3[:2] == ['val', 'val_']
+    assert cfg.val1.val4[:2] == ['val', 'val_']
+    raises(PropertiesOptionError, "cfg.val1.val3[2]")
+    raises(PropertiesOptionError, "cfg.val1.val4[2]")
+    assert cfg.val1.val1 == ['val', 'val_', 'val3']
+
+
+def test_callback_master_and_slaves_master_mandatory3():
+    val = StrOption('val', "", default='val')
+    val_ = StrOption('val_', "", default='val_')
+    val1 = StrOption('val1', "", multi=True, callback=return_value2, callback_params={'': ((val, False),), 'val': ((val_, False),)}, properties=('mandatory',))
+    val3 = StrOption('val3', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)}, properties=('mandatory',))
+    val4 = StrOption('val4', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)}, properties=('mandatory',))
+    interface1 = OptionDescription('val1', '', [val1, val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [val, val_, interface1])
+    cfg = Config(maconfig)
+    cfg.read_only()
+    assert cfg.val1.val3 == ['val', 'val_']
+    assert cfg.val1.val4 == ['val', 'val_']
+    assert cfg.val1.val1 == ['val', 'val_']
+    cfg.read_write()
+    cfg.val1.val1.append('val3')
+    cfg.read_only()
+    assert cfg.val1.val3 == ['val', 'val_', 'val3']
+    assert cfg.val1.val4 == ['val', 'val_', 'val3']
+    assert cfg.val1.val1 == ['val', 'val_', 'val3']
+
+
+def test_callback_master_and_slaves_master_mandatory4():
+    val = StrOption('val', "", default='val')
+    val1 = StrOption('val1', "", multi=True, callback=return_value2, callback_params={'': ((val, False),)}, properties=('mandatory',))
+    val3 = StrOption('val3', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)}, properties=('mandatory',))
+    val4 = StrOption('val4', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)}, properties=('mandatory',))
+    interface1 = OptionDescription('val1', '', [val1, val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [val, interface1])
+    cfg = Config(maconfig)
+    cfg.read_only()
+    assert cfg.val1.val3 == ['val']
+    assert cfg.val1.val4 == ['val']
+    assert cfg.val1.val1 == ['val']
+    cfg.read_write()
+    cfg.val1.val1.append('val3')
+    cfg.read_only()
+    assert cfg.val1.val1 == ['val', 'val3']
+    assert cfg.val1.val3 == ['val', 'val3']
+    assert cfg.val1.val4 == ['val', 'val3']
+
+
 def test_callback_master_and_slaves_master3():
     val1 = StrOption('val1', "", multi=True, properties=('mandatory', 'empty'))
     val2 = StrOption('val2', "", multi=True, default_multi='val2', properties=('expert',))
@@ -477,10 +604,10 @@ def test_callback_master_and_slaves_master4():
     cfg.read_write()
     cfg.cfgimpl_get_settings().append('expert')
     cfg.cfgimpl_get_settings().setpermissive(('expert',))
-    assert list(cfg.cfgimpl_get_values().mandatory_warnings(force_permissive=True)) == []
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings()) == []
 
 
-def test_callback_master_and_slaves_master_mandatory_transitive():
+def test_consistency_master_and_slaves_master_mandatory_transitive():
     #default value
     val1 = IPOption('val1', "", ['192.168.0.1'], multi=True, properties=('mandatory',))
     val2 = NetmaskOption('val2', "", multi=True, default_multi='255.255.255.0', properties=('disabled', 'mandatory'))
@@ -498,10 +625,10 @@ def test_callback_master_and_slaves_master_mandatory_transitive():
     cfg.read_write()
     raises(PropertiesOptionError, "cfg.val1.val1")
     raises(PropertiesOptionError, "cfg.val3.val3")
-    assert list(cfg.cfgimpl_get_values().mandatory_warnings(force_permissive=True)) == []
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings()) == []
 
 
-def test_callback_master_and_slaves_master_mandatory_non_transitive():
+def test_consistency_master_and_slaves_master_mandatory_non_transitive():
     #no value
     val1 = IPOption('val1', "", multi=True, properties=('mandatory',))
     val2 = NetmaskOption('val2', "", multi=True, default_multi='255.255.255.0', properties=('disabled', 'mandatory'))
@@ -517,7 +644,7 @@ def test_callback_master_and_slaves_master_mandatory_non_transitive():
     maconfig = OptionDescription('rootconfig', '', [interface1, interface2])
     cfg = Config(maconfig)
     cfg.read_write()
-    assert list(cfg.cfgimpl_get_values().mandatory_warnings(force_permissive=True)) == ["val1.val1"]
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings()) == ["val1.val1"]
 
 
 def test_callback_master_and_slaves_master_list():
@@ -944,3 +1071,21 @@ def test_re_set_callback():
     st2 = StrOption('st2', "", multi=True)
     st2.impl_set_callback(return_value, {'': ((st1, False),)})
     raises(ConfigError, "st2.impl_set_callback(return_value, {'': ((st1, False),)})")
+
+
+def test_callback_raise():
+    opt1 = BoolOption('opt1', 'Option 1', callback=return_raise)
+    opt2 = BoolOption('opt2', 'Option 2', callback=return_valueerror)
+    od1 = OptionDescription('od1', '', [opt1])
+    od2 = OptionDescription('od2', '', [opt2])
+    maconfig = OptionDescription('rootconfig', '', [od1, od2])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    try:
+        cfg.od1.opt1
+    except ConfigError as err:
+        assert '"Option 1"' in str(err)
+    try:
+        cfg.od2.opt2
+    except ConfigError as err:
+        assert '"Option 2"' in str(err)
