@@ -1,11 +1,11 @@
 "test all types of option default values for options, add new option in a descr"
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
 
 from tiramisu.config import Config
-from tiramisu.setting import owners
+from tiramisu.setting import owners, groups
 from tiramisu.error import PropertiesOptionError, ConfigError
 from tiramisu.option import IntOption, FloatOption, StrOption, ChoiceOption, \
     BoolOption, OptionDescription
@@ -113,6 +113,65 @@ def test_force_default_on_freeze_multi():
     raises(ConfigError, "config.cfgimpl_get_values().setowner(dummy1, owners.frozenmulti)")
 
 
+def test_force_default_on_freeze_master():
+    dummy1 = BoolOption('dummy1', 'Test int option', multi=True, properties=('force_default_on_freeze',))
+    dummy2 = BoolOption('dummy2', 'Test string option', multi=True)
+    descr = OptionDescription("dummy1", "", [dummy1, dummy2])
+    descr.impl_set_group_type(groups.master)
+    descr = OptionDescription("root", "", [descr])
+    raises(ConfigError, "Config(descr)")
+
+
+def test_force_default_on_freeze_master_frozen():
+    dummy1 = BoolOption('dummy1', 'Test int option', multi=True, properties=('force_default_on_freeze', 'frozen'))
+    dummy2 = BoolOption('dummy2', 'Test string option', multi=True)
+    descr = OptionDescription("dummy1", "", [dummy1, dummy2])
+    descr.impl_set_group_type(groups.master)
+    descr = OptionDescription("root", "", [descr])
+    config = Config(descr)
+    setting = config.cfgimpl_get_settings()
+    raises(ConfigError, "setting[dummy1].remove('frozen')")
+
+
+def test_force_default_on_freeze_slave():
+    dummy1 = BoolOption('dummy1', 'Test int option', multi=True)
+    dummy2 = BoolOption('dummy2', 'Test string option', multi=True, properties=('force_default_on_freeze',))
+    descr = OptionDescription("dummy1", "", [dummy1, dummy2])
+    descr.impl_set_group_type(groups.master)
+    descr = OptionDescription("root", "", [descr])
+    config = Config(descr)
+    config.read_write()
+    owners.addowner("frozenmultislave")
+    config.dummy1.dummy1.append(True)
+    config.dummy1.dummy2[0] = False
+    assert config.dummy1.dummy1 == [True]
+    assert config.dummy1.dummy2 == [False]
+    assert config.getowner(dummy1) == owners.user
+    assert config.getowner(dummy2, 0) == owners.user
+    setting = config.cfgimpl_get_settings()
+    setting[dummy2].append('frozen')
+    assert config.dummy1.dummy1 == [True]
+    assert config.dummy1.dummy2 == [None]
+    assert config.getowner(dummy1) == owners.user
+    assert config.getowner(dummy2) == owners.default
+    raises(PropertiesOptionError, "config.cfgimpl_get_values().setowner(dummy2, owners.frozenmulti)")
+    #
+    setting[dummy2].remove('frozen')
+    config.dummy1.dummy1.append(True)
+    config.dummy1.dummy2[1] = False
+    assert config.dummy1.dummy1 == [True, True]
+    assert config.dummy1.dummy2 == [False, False]
+    setting[dummy2].append('frozen')
+    assert config.dummy1.dummy1 == [True, True]
+    assert config.dummy1.dummy2 == [None, None]
+    assert config.dummy1.dummy1.pop(1)
+    assert config.dummy1.dummy1 == [True]
+    assert config.dummy1.dummy2 == [None]
+    setting[dummy2].remove('frozen')
+    assert config.dummy1.dummy1 == [True]
+    assert config.dummy1.dummy2 == [False]
+
+
 def test_overrides_changes_option_value():
     "with config.override(), the default is changed and the value is changed"
     descr = OptionDescription("test", "", [
