@@ -1,9 +1,11 @@
 "config.set() or config.setoption() or option.setoption()"
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
 
+from tiramisu.i18n import _
+from tiramisu.error import display_list
 from tiramisu.setting import owners, groups
 from tiramisu.config import Config
 from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption, \
@@ -174,6 +176,22 @@ def test_multi_with_requires_in_another_group():
     assert 'hidden' in setting[stroption]
 
 
+def test_multi_with_requires_in_another_group_inverse():
+    s = StrOption("string", "", default=["string"], multi=True)
+    intoption = IntOption('int', 'Test int option', default=0)
+    stroption = StrOption('str', 'Test string option', default=["abc"],
+            requires=[{'option': intoption, 'expected': 0, 'action': 'hidden', 'inverse': True}], multi=True)
+    descr = OptionDescription("opt", "", [stroption])
+    descr2 = OptionDescription("opt2", "", [intoption, s, descr])
+    config = Config(descr2)
+    setting = config.cfgimpl_get_settings()
+    config.read_write()
+    assert not 'hidden' in setting[stroption]
+    config.int = 1
+    raises(PropertiesOptionError,  "config.opt.str = ['a', 'b']")
+    assert 'hidden' in setting[stroption]
+
+
 def test_apply_requires_from_config():
     s = StrOption("string", "", default=["string"], multi=True)
     intoption = IntOption('int', 'Test int option', default=0)
@@ -230,6 +248,14 @@ def test_multi_with_requires_that_is_multi():
     raises(ValueError, "Config(descr)")
 
 
+def test_multi_with_requires_that_is_multi_inverse():
+    b = IntOption('int', 'Test int option', default=[0], multi=True)
+    c = StrOption('str', 'Test string option', default=['abc'], requires=[{'option': b, 'expected': 0, 'action': 'hidden', 'inverse': True}], multi=True)
+    descr = OptionDescription("opt", "", [b, c])
+    descr
+    raises(ValueError, "Config(descr)")
+
+
 def test_multi_with_requires_that_is_masterslave():
     b = IntOption('int', 'Test int option', default=[0], multi=True)
     c = StrOption('str', 'Test string option', requires=[{'option': b, 'expected': 1, 'action': 'hidden'}], multi=True)
@@ -246,13 +272,50 @@ def test_multi_with_requires_that_is_masterslave_master():
     raises(ValueError, "Config(descr)")
 
 
-def test_multi_with_requires_that_is_masterslave2():
+def test_multi_with_requires_that_is_masterslave_slave():
     b = IntOption('int', 'Test int option', default=[0], multi=True)
     c = StrOption('str', 'Test string option', multi=True)
     d = StrOption('str1', 'Test string option', requires=[{'option': c, 'expected': '1', 'action': 'hidden'}], multi=True)
     descr = OptionDescription("int", "", [b, c, d])
     descr.impl_set_group_type(groups.master)
-    Config(descr)
+    config = Config(descr)
+    config.read_write()
+    assert config.int == [0]
+    assert config.str == [None]
+    assert config.str1 == [None]
+    config.int = [0, 1]
+    assert config.int == [0, 1]
+    assert config.str == [None, None]
+    assert config.str1 == [None, None]
+    config.str = [None, '1']
+    config.read_only()
+    assert config.str1 == [None, None]
+    config.read_write()
+    assert config.str1[0] is None
+    raises(PropertiesOptionError, 'config.str1[1]')
+
+
+def test_multi_with_requires_that_is_masterslave_slave_inverse():
+    b = IntOption('int', 'Test int option', default=[0], multi=True)
+    c = StrOption('str', 'Test string option', multi=True)
+    d = StrOption('str1', 'Test string option', requires=[{'option': c, 'expected': None, 'action': 'hidden', 'inverse': True}], multi=True)
+    descr = OptionDescription("int", "", [b, c, d])
+    descr.impl_set_group_type(groups.master)
+    config = Config(descr)
+    config.read_write()
+    assert config.int == [0]
+    assert config.str == [None]
+    assert config.str1 == [None]
+    config.int = [0, 1]
+    assert config.int == [0, 1]
+    assert config.str == [None, None]
+    assert config.str1 == [None, None]
+    config.str = [None, '1']
+    config.read_only()
+    assert config.str1 == [None, None]
+    config.read_write()
+    assert config.str1[0] is None
+    raises(PropertiesOptionError, 'config.str1[1]')
 
 
 def test_multi_with_requires_that_is_not_same_masterslave():
@@ -471,3 +534,94 @@ def test_reset_properties_force_store_value():
     setting.reset(all_properties=True)
     assert setting._p_.get_modified_properties() == {}
     raises(ValueError, 'setting.reset(all_properties=True, opt=option)')
+
+
+def test_pprint():
+    msg_error = _("cannot access to {0} \"{1}\" because has {2} {3}")
+    msg_is = _('the value of "{0}" is "{1}"')
+    msg_is_not = _('the value of "{0}" is not "{1}"')
+    properties = _('properties')
+    prop = _('property')
+
+    s = StrOption("string", "", default=["string"], default_multi="string", multi=True, properties=('hidden', 'disabled'))
+    s2 = StrOption("string2", "", default="string")
+    s3 = StrOption("string3", "", default=["string"], default_multi="string", multi=True, properties=('hidden',))
+    intoption = IntOption('int', 'Test int option', default=0)
+    stroption = StrOption('str', 'Test string option', default="abc",
+                          requires=[{'option': intoption, 'expected': 2, 'action': 'hidden', 'inverse': True},
+                    {'option': intoption, 'expected': 3, 'action': 'hidden', 'inverse': True},
+                    {'option': intoption, 'expected': 4, 'action': 'hidden', 'inverse': True},
+                    {'option': intoption, 'expected': 1, 'action': 'disabled'},
+                    {'option': s2, 'expected': 'string', 'action': 'disabled'}])
+
+    val2 = StrOption('val2', "")
+    descr2 = OptionDescription("options", "", [val2], requires=[{'option': intoption, 'expected': 1, 'action': 'hidden'}])
+
+    val3 = StrOption('val3', "", requires=[{'option': stroption, 'expected': '2', 'action': 'hidden', 'inverse': True}])
+
+    descr = OptionDescription("options", "", [s, s2, s3, intoption, stroption, descr2, val3])
+    config = Config(descr)
+    setting = config.cfgimpl_get_settings()
+    config.read_write()
+    config.int = 1
+    try:
+        config.str
+    except Exception as error:
+        err = error
+        pass
+
+    list_disabled_1 = 'disabled (' + display_list([msg_is.format('string2', 'string'), msg_is.format('Test int option', '1')]) + ')'
+    list_disabled_2 = 'disabled (' + display_list([msg_is.format('Test int option', '1'), msg_is.format('string2', 'string')]) + ')'
+    list_hidden = 'hidden (' + msg_is_not.format('Test int option', display_list([2, 3, 4], 'or')) + ')'
+    comp1 = str(err) == _(msg_error.format('option', 'Test string option', properties, display_list([list_disabled_1, list_hidden])))
+    comp2 = str(err) == _(msg_error.format('option', 'Test string option', properties, display_list([list_disabled_2, list_hidden])))
+    comp3 = str(err) == _(msg_error.format('option', 'Test string option', properties, display_list([list_hidden, list_disabled_1])))
+    comp4 = str(err) == _(msg_error.format('option', 'Test string option', properties, display_list([list_hidden, list_disabled_2])))
+    assert comp1 or comp2 or comp3 or comp4
+
+    try:
+        config.options.val2
+    except Exception as error:
+        err = error
+        pass
+
+    assert str(err) == msg_error.format('optiondescription', 'options', prop, 'hidden (' + msg_is.format('Test int option', 1) + ')')
+
+    try:
+        config.val3
+    except Exception as error:
+        err = error
+        pass
+    msg_1 = msg_is.format('string2', 'string')
+    msg_2 = msg_is.format('Test int option', 1)
+    msg_3 = msg_is_not.format('Test int option', display_list([2, 3, 4], 'or'))
+
+    list_hidden = 'hidden (' + display_list([msg_1, msg_2, msg_3]) + ')'
+    comp1 = str(err) == msg_error.format('option', 'val3', prop, list_hidden)
+    list_hidden = 'hidden (' + display_list([msg_1, msg_3, msg_2]) + ')'
+    comp2 = str(err) == msg_error.format('option', 'val3', prop, list_hidden)
+    list_hidden = 'hidden (' + display_list([msg_2, msg_1, msg_3]) + ')'
+    comp3 = str(err) == msg_error.format('option', 'val3', prop, list_hidden)
+    list_hidden = 'hidden (' + display_list([msg_2, msg_3, msg_1]) + ')'
+    comp4 = str(err) == msg_error.format('option', 'val3', prop, list_hidden)
+    list_hidden = 'hidden (' + display_list([msg_3, msg_1, msg_2]) + ')'
+    comp5 = str(err) == msg_error.format('option', 'val3', prop, list_hidden)
+    list_hidden = 'hidden (' + display_list([msg_3, msg_2, msg_1]) + ')'
+    comp6 = str(err) == msg_error.format('option', 'val3', prop, list_hidden)
+    assert comp1 or comp2 or comp3 or comp4 or comp5 or comp6
+
+    try:
+        config.string
+    except Exception as error:
+        err = error
+        pass
+
+    assert str(err) == msg_error.format('option', 'string', properties, display_list(['disabled', 'hidden']))
+
+    try:
+        config.string3
+    except Exception as error:
+        err = error
+        pass
+
+    assert str(err) == msg_error.format('option', 'string3', prop, 'hidden')
