@@ -1,13 +1,13 @@
 # coding: utf-8
 "frozen and hidden values"
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
 
 from tiramisu.config import Config
 from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption, \
-    StrOption, OptionDescription
+    PasswordOption, StrOption, DateOption, OptionDescription
 from tiramisu.error import PropertiesOptionError
 
 
@@ -133,3 +133,28 @@ def test_with_many_subgroups():
     assert name == "booltwo"
     getattr(homeconfig.cfgimpl_get_description(), name)
     assert 'hidden' in setting[booltwo]
+
+
+def test_password_option():
+    o = PasswordOption('o', '')
+    d = OptionDescription('d', '', [o])
+    c = Config(d)
+
+    c.o = u'a_valid_password'
+    raises(ValueError, "c.o = 1")
+
+
+def test_date_option():
+    o = DateOption('o', '')
+    d = OptionDescription('d', '', [o])
+    c = Config(d)
+
+    c.o = u'2017-02-04'
+    c.o = u'2017-2-4'
+    raises(ValueError, "c.o = 1")
+    raises(ValueError, "c.o = u'2017-13-20'")
+    raises(ValueError, "c.o = u'2017-11-31'")
+    raises(ValueError, "c.o = u'2017-12-32'")
+    raises(ValueError, "c.o = u'2017-2-29'")
+    raises(ValueError, "c.o = u'2-2-2017'")
+    raises(ValueError, "c.o = u'2017/2/2'")
