@@ -1,5 +1,5 @@
 "configuration objects global API"
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
