@@ -1,5 +1,5 @@
 #this test is much more to test that **it's there** and answers attribute access
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
