@@ -1,5 +1,5 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from tiramisu.setting import groups, owners
@@ -206,7 +206,7 @@ def test_groups_with_master_hidden_in_config2():
     raises(PropertiesOptionError, "cfg.getattr('netmask_admin_eth0')")
     cfg.ip_admin_eth0.append('192.168.1.1')
     assert cfg.ip_admin_eth0 == ['192.168.1.1']
-    raises(PropertiesOptionError, "cfg.netmask_admin_eth0")
+    raises(PropertiesOptionError, "cfg.netmask_admin_eth0[0]")
     del(cfg.ip_admin_eth0)
     assert cfg.ip_admin_eth0 == []
     #del
@@ -263,6 +263,28 @@ def test_values_with_master_disabled_master():
     raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[0] = '192.168.230.145'")
 
 
+def test_values_with_master_remove():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.230.145')
+    cfg.ip_admin_eth0.ip_admin_eth0.remove('192.168.230.145')
+    raises(ValueError, "cfg.ip_admin_eth0.ip_admin_eth0.remove('192.168.230.14')")
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.230.15')
+    cfg.ip_admin_eth0.netmask_admin_eth0[-1] = '255.255.255.0'
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.230.16')
+    cfg.ip_admin_eth0.netmask_admin_eth0[-1] = '255.255.255.128'
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.230.15', '192.168.230.16']
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == ['255.255.255.0', '255.255.255.128']
+    cfg.ip_admin_eth0.ip_admin_eth0.remove('192.168.230.15')
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.230.16']
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == ['255.255.255.128']
+
+
 def test_master_not_valid_name():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
@@ -341,6 +363,13 @@ def test_reset_values_with_master_and_slaves():
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
 
 
+def test_reset_values_with_master_and_slaves_default_value():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True, default=['192.168.230.145'])
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True, default=['255.255.255.0'])
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    raises(ValueError, "interface1.impl_set_group_type(groups.master)")
+
+
 def test_reset_values_with_master_and_slaves_default():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True, default=['192.168.230.145'])
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
@@ -461,6 +490,77 @@ def test_values_with_master_and_slaves_master():
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
 
 
+def test_values_with_master_and_slaves_master_pop():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.146")
+    cfg.ip_admin_eth0.netmask_admin_eth0[1] = '255.255.0.0'
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ["192.168.230.145", "192.168.230.146"]
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None, '255.255.0.0']
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.145', '192.168.230.146')),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'1': 'user'},
+                                                                  {'1': '255.255.0.0'})}
+    cfg.ip_admin_eth0.ip_admin_eth0.pop(0)
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.146',)),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'0': 'user'},
+                                                                  {'0': '255.255.0.0'})}
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ["192.168.230.146"]
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == ['255.255.0.0']
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.146")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.147")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.148")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.149")
+    cfg.ip_admin_eth0.netmask_admin_eth0[3] = '255.255.0.0'
+    cfg.ip_admin_eth0.netmask_admin_eth0[4] = '255.255.0.0'
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.146', "192.168.230.145", "192.168.230.146", "192.168.230.147", "192.168.230.148", "192.168.230.149")),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'0': 'user', '3': 'user', '4': 'user'},
+                                                                  {'0': '255.255.0.0', '3': '255.255.0.0', '4': '255.255.0.0'})}
+    cfg.ip_admin_eth0.ip_admin_eth0.pop(5)
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.146', "192.168.230.145", "192.168.230.146", "192.168.230.147", "192.168.230.148")),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'0': 'user', '3': 'user', '4': 'user'},
+                                                                  {'0': '255.255.0.0', '3': '255.255.0.0', '4': '255.255.0.0'})}
+    cfg.ip_admin_eth0.ip_admin_eth0.pop(2)
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.146', "192.168.230.145", "192.168.230.147", "192.168.230.148")),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'0': 'user', '2': 'user', '3': 'user'},
+                                                                  {'0': '255.255.0.0', '2': '255.255.0.0', '3': '255.255.0.0'})}
+    cfg.ip_admin_eth0.ip_admin_eth0.pop(2)
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.146', "192.168.230.145", "192.168.230.148")),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'0': 'user', '2': 'user'},
+                                                                  {'0': '255.255.0.0', '2': '255.255.0.0'})}
+    cfg.ip_admin_eth0.ip_admin_eth0.pop(2)
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': (
+                                                                  'user',
+                                                                  ('192.168.230.146', "192.168.230.145")),
+                                                              'ip_admin_eth0.netmask_admin_eth0': (
+                                                                  {'0': 'user'},
+                                                                  {'0': '255.255.0.0'})}
+
+
 def test_values_with_master_and_slaves_master_error():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
@@ -647,3 +747,19 @@ def test_multi_master_default_slave():
     cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.1')
     cfg.cfgimpl_reset_cache()
     assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.1.1']
+
+
+def test_groups_with_master_get_modified_value():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert cfg.cfgimpl_get_values().get_modified_values() == {}
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.1')
+    cfg.cfgimpl_reset_cache()
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': ('user', ('192.168.1.1',))}
+    cfg.ip_admin_eth0.netmask_admin_eth0 = ['255.255.255.255']
+    assert cfg.cfgimpl_get_values().get_modified_values() == {'ip_admin_eth0.ip_admin_eth0': ('user', ('192.168.1.1',)), 'ip_admin_eth0.netmask_admin_eth0': ({'0': 'user'}, {'0': '255.255.255.255'})}
