@@ -1,10 +1,10 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
 
-from tiramisu.option import IntOption, OptionDescription
+from tiramisu.option import IntOption, UnicodeOption, OptionDescription
 from tiramisu.config import Config
 from tiramisu.error import PropertiesOptionError
 
@@ -199,3 +199,16 @@ def test_invalid_option_permissive():
     setting = config.cfgimpl_get_settings()
     config.read_write()
     raises(TypeError, "setting.setpermissive(['frozen', 'disabled',], u1)")
+
+
+def test_remove_option_permissive():
+    var1 = UnicodeOption('var1', '', u'value', properties=('hidden',))
+    od1 = OptionDescription('od1', '', [var1])
+    rootod = OptionDescription('rootod', '', [od1])
+    config = Config(rootod)
+    config.read_write()
+    raises(PropertiesOptionError, "config.od1.var1")
+    config.cfgimpl_get_settings().setpermissive(('hidden',), var1)
+    assert config.od1.var1 == u'value'
+    config.cfgimpl_get_settings().setpermissive(tuple(), var1)
+    raises(PropertiesOptionError, "config.od1.var1")
