@@ -1,5 +1,5 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from copy import copy
@@ -349,6 +349,31 @@ def test_requires_not_transitive():
     c.ip_address_service_web
 
 
+def test_requires_not_transitive_not_same_action():
+    a = BoolOption('activate_service', '', True)
+    b = BoolOption('activate_service_web', '', True,
+                   requires=[{'option': a, 'expected': False, 'action': 'disabled'}])
+    d = IPOption('ip_address_service_web', '',
+                 requires=[{'option': b, 'expected': False,
+                            'action': 'hidden', 'transitive': False}])
+    od = OptionDescription('service', '', [a, b, d])
+    c = Config(od)
+    c.read_write()
+    c.activate_service
+    c.activate_service_web
+    c.ip_address_service_web
+    c.activate_service = False
+    #
+    props = []
+    try:
+        c.activate_service_web
+    except PropertiesOptionError as err:
+        props = err.proptype
+    assert props == ['disabled']
+    #
+    raises(RequirementError, "c.ip_address_service_web")
+
+
 def test_requires_None():
     a = BoolOption('activate_service', '')
     b = IPOption('ip_address_service', '',
@@ -570,8 +595,35 @@ def test_requires_requirement_append():
 
 def test_requires_different_inverse():
     a = BoolOption('activate_service', '', True)
-    a
-    raises(ValueError, "IPOption('ip_address_service', '', requires=[{'option': a, 'expected': True, 'action': 'disabled', 'inverse': True}, {'option': a, 'expected': True, 'action': 'disabled', 'inverse': False}])")
+    b = IPOption('ip_address_service', '', requires=[
+        {'option': a, 'expected': True, 'action': 'disabled', 'inverse': True},
+        {'option': a, 'expected': True, 'action': 'disabled', 'inverse': False}])
+    od = OptionDescription('service', '', [a, b])
+    c = Config(od)
+    c.read_write()
+    raises(PropertiesOptionError, 'c.ip_address_service')
+    c.activate_service = False
+    raises(PropertiesOptionError, 'c.ip_address_service')
+
+
+def test_requires_different_inverse_unicode():
+    a = BoolOption('activate_service', '', True)
+    d = StrOption('activate_other_service', '', 'val2')
+    b = IPOption('ip_address_service', '', requires=[
+        {'option': a, 'expected': True, 'action': 'disabled', 'inverse': True},
+        {'option': d, 'expected': 'val1', 'action': 'disabled', 'inverse': False}])
+    od = OptionDescription('service', '', [a, d, b])
+    c = Config(od)
+    c.read_write()
+    assert c.ip_address_service == None
+    c.activate_service = False
+    raises(PropertiesOptionError, 'c.ip_address_service')
+    c.activate_service = True
+    assert c.ip_address_service == None
+    c.activate_other_service = 'val1'
+    raises(PropertiesOptionError, 'c.ip_address_service')
+    c.activate_service = False
+    raises(PropertiesOptionError, 'c.ip_address_service')
 
 
 def test_requires_recursive_path():
@@ -644,6 +696,12 @@ def test_master_slave_requires():
     cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.1')
     assert cfg.ip_admin_eth0.netmask_admin_eth0[0] is None
     raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[1]")
+    cfg.ip_admin_eth0.ip_admin_eth0[1] = '192.168.1.2'
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[1] is None
+    cfg.ip_admin_eth0.netmask_admin_eth0[1] = '255.255.255.255'
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[1] == '255.255.255.255'
+    cfg.ip_admin_eth0.ip_admin_eth0[1] = '192.168.1.1'
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[1]")
 
 
 def test_master_slave_requires_no_master():
@@ -668,3 +726,11 @@ def test_master_slave_requires_no_master():
     raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0")
     raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[0]")
     raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[1]")
+    cfg.activate = True
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None, None]
+    cfg.ip_admin_eth0.netmask_admin_eth0 = [None, '255.255.255.255']
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[1] == '255.255.255.255'
+    cfg.activate = False
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0")
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[0]")
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[1]")
