@@ -1,5 +1,5 @@
 ## coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from py.test import raises
@@ -11,6 +11,9 @@ from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption,\
     URLOption, FilenameOption
 
 
+print("FIXME slot pour un masterslaves !!")
+
+
 def test_slots_option():
     c = ChoiceOption('a', '', ('a',))
     raises(AttributeError, "c.x = 1")
