@@ -1,4 +1,4 @@
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from tiramisu.option import BoolOption, UnicodeOption, SymLinkOption, \
@@ -20,7 +20,7 @@ def return_value(value=None):
 def _get_slots(opt):
     slots = set()
     for subclass in opt.__class__.__mro__:
-        if subclass is not object:
+        if subclass is not object and '__slots__' in dir(subclass):
             slots.update(subclass.__slots__)
     return slots
 
@@ -107,9 +107,9 @@ def _diff_opt(opt1, opt2):
                         else:
                             assert v[1] == val2[idx][1]
         elif attr == '_master_slaves':
-            assert val1.master.impl_getname() == val2.master.impl_getname()
-            sval1 = [opt.impl_getname() for opt in val1.slaves]
-            sval2 = [opt.impl_getname() for opt in val2.slaves]
+            assert val1._p_._sm_getmaster().impl_getname() == val2._p_._sm_getmaster().impl_getname()
+            sval1 = [opt.impl_getname() for opt in val1._p_._sm_getslaves()]
+            sval2 = [opt.impl_getname() for opt in val2._p_._sm_getslaves()]
             assert sval1 == sval2
         elif attr == '_subdyn':
             try:
@@ -214,14 +214,18 @@ def test_diff_information_config():
     except ValueError:
         cfg = Config(o1, session_id='29090938')
         cfg._impl_test = True
-    cfg.impl_set_information('info', 'oh')
+        cfg.impl_set_information('info', 'oh')
 
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
-    _diff_conf(cfg, q)
-    assert cfg.impl_get_information('info') == 'oh'
-    assert q.impl_get_information('info') == 'oh'
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
+        _diff_conf(cfg, q)
+        assert cfg.impl_get_information('info') == 'oh'
+        assert q.impl_get_information('info') == 'oh'
+    try:
+        delete_session('config', '29090938')
+    except ValueError:
+        pass
 
 
 def test_diff_opt_multi():
@@ -295,10 +299,10 @@ def test_state_config():
     except ValueError:
         cfg = Config(maconfig, session_id='29090931')
         cfg._impl_test = True
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
-    _diff_conf(cfg, q)
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
+        _diff_conf(cfg, q)
     try:
         delete_session('config', '29090931')
     except ValueError:
@@ -324,10 +328,10 @@ def test_state_config2():
     except ValueError:
         cfg = Config(od2, session_id='29090939')
         cfg._impl_test = True
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
-    _diff_conf(cfg, q)
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
+        _diff_conf(cfg, q)
     try:
         delete_session('config', '29090939')
     except ValueError:
@@ -349,10 +353,14 @@ def test_diff_opt_config():
         cfg = Config(o1, session_id='29090940')
         cfg._impl_test = True
 
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
-    _diff_conf(cfg, q)
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_opts(cfg.cfgimpl_get_description(), q.cfgimpl_get_description())
+        _diff_conf(cfg, q)
+    try:
+        delete_session('config', '29090940')
+    except ValueError:
+        pass
 
 
 def test_state_properties():
@@ -363,11 +371,11 @@ def test_state_properties():
     except ValueError:
         cfg = Config(maconfig, session_id='29090932')
         cfg._impl_test = True
-    cfg.read_write()
-    cfg.cfgimpl_get_settings()[val1].append('test')
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_conf(cfg, q)
+        cfg.read_write()
+        cfg.cfgimpl_get_settings()[val1].append('test')
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_conf(cfg, q)
     try:
         delete_session('config', '29090932')
     except ValueError:
@@ -382,13 +390,13 @@ def test_state_values():
     except ValueError:
         cfg = Config(maconfig, session_id='29090933')
         cfg._impl_test = True
-    cfg.val1 = True
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_conf(cfg, q)
-    q.val1 = False
-    assert cfg.val1 is True
-    assert q.val1 is False
+        cfg.val1 = True
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_conf(cfg, q)
+        q.val1 = False
+        assert cfg.val1 is True
+        assert q.val1 is False
     try:
         delete_session('config', '29090933')
     except ValueError:
@@ -403,15 +411,15 @@ def test_state_values_owner():
     except ValueError:
         cfg = Config(maconfig, session_id='29090934')
         cfg._impl_test = True
-    owners.addowner('newowner')
-    cfg.cfgimpl_get_settings().setowner(owners.newowner)
-    cfg.val1 = True
-    a = dumps(cfg)
-    q = loads(a)
-    _diff_conf(cfg, q)
-    q.val1 = False
-    nval1 = q.cfgimpl_get_description().val1
-    assert q.getowner(nval1) == owners.newowner
+        owners.addowner('newowner')
+        cfg.cfgimpl_get_settings().setowner(owners.newowner)
+        cfg.val1 = True
+        a = dumps(cfg)
+        q = loads(a)
+        _diff_conf(cfg, q)
+        q.val1 = False
+        nval1 = q.cfgimpl_get_description().val1
+        assert q.getowner(nval1) == owners.newowner
     try:
         delete_session('config', '29090934')
     except ValueError:
@@ -422,17 +430,18 @@ def test_state_metaconfig():
     i1 = IntOption('i1', '')
     od1 = OptionDescription('od1', '', [i1])
     od2 = OptionDescription('od2', '', [od1])
-    conf1 = Config(od2, session_id='29090935')
-    conf1._impl_test = True
-    conf2 = Config(od2, session_id='29090936')
-    conf2._impl_test = True
-    meta = MetaConfig([conf1, conf2], session_id='29090937')
-    meta._impl_test = True
-    raises(ConfigError, "dumps(meta)")
+    try:
+        cfg = Config(od2, persistent=True, session_id='29090935')
+    except ValueError:
+        conf1 = Config(od2, session_id='29090935')
+        conf1._impl_test = True
+        conf2 = Config(od2, session_id='29090936')
+        conf2._impl_test = True
+        meta = MetaConfig([conf1, conf2], session_id='29090937')
+        meta._impl_test = True
+        raises(ConfigError, "dumps(meta)")
     try:
         delete_session('config', '29090935')
-        delete_session('config', '29090936')
-        delete_session('config', '29090937')
     except ValueError:
         pass
 
@@ -441,19 +450,20 @@ def test_state_groupconfig():
     i1 = IntOption('i1', '')
     od1 = OptionDescription('od1', '', [i1])
     od2 = OptionDescription('od2', '', [od1])
-    conf1 = Config(od2, session_id='29090935')
-    conf1._impl_test = True
-    conf2 = Config(od2, session_id='29090936')
-    conf2._impl_test = True
-    meta = GroupConfig([conf1, conf2], session_id='29090937')
-    meta._impl_test = True
-    a = dumps(meta)
-    q = loads(a)
-    _diff_conf(meta, q)
     try:
-        delete_session('config', '29090935')
-        delete_session('config', '29090936')
-        delete_session('config', '29090937')
+        cfg = Config(od2, persistent=True, session_id='29090938')
+    except ValueError:
+        conf1 = Config(od2, session_id='29090938')
+        conf1._impl_test = True
+        conf2 = Config(od2, session_id='29090939')
+        conf2._impl_test = True
+        meta = GroupConfig([conf1, conf2], session_id='29090940')
+        meta._impl_test = True
+        a = dumps(meta)
+        q = loads(a)
+        _diff_conf(meta, q)
+    try:
+        delete_session('config', '29090938')
     except ValueError:
         pass
 
