@@ -1,10 +1,10 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from tiramisu.setting import groups, owners
-from tiramisu.config import Config
-from tiramisu.option import StrOption, OptionDescription, submulti
+from tiramisu.config import Config, MetaConfig
+from tiramisu.option import StrOption, IntOption, OptionDescription, submulti
 from tiramisu.value import SubMulti, Multi
 from tiramisu.error import SlaveError
 
@@ -663,3 +663,36 @@ def test_callback_submulti():
     assert cfg.getowner(multi2) == owners.default
     assert cfg.multi == [['val']]
     assert cfg.multi2 == [['val']]
+
+
+def test_submulti_unique():
+    i = IntOption('int', '', multi=submulti, unique=True)
+    o = OptionDescription('od', '', [i])
+    c = Config(o)
+    assert c.int == []
+    c.int = [[0]]
+    assert c.int == [[0]]
+    raises(ValueError, "c.int = [[0, 0]]")
+    c.int = [[0], [0]]
+    raises(ValueError, "c.int[0] = [1, 0, 2, 3, 4, 5, 6, 0, 7]")
+    raises(ValueError, "c.int[0].append(0)")
+    raises(ValueError, "c.int[0].extend([1, 2, 1, 3])")
+    raises(ValueError, "c.int[0].extend([1, 2, 0, 3])")
+    c.int[0].extend([4, 5, 6])
+
+
+def test_multi_submulti_meta():
+    multi = StrOption('multi', '', multi=submulti)
+    od = OptionDescription('od', '', [multi])
+    conf1 = Config(od, name='conf1')
+    conf1.read_write()
+    conf2 = Config(od, name='conf2')
+    conf2.read_write()
+    meta = MetaConfig([conf1, conf2])
+    meta.read_write()
+    meta.multi = [['val']]
+    assert meta.multi == [['val']]
+    multi = conf1.multi[0]
+    multi.append()
+    assert conf1.multi == [['val', None]]
+    assert meta.multi == [['val']]
