@@ -1,5 +1,5 @@
 # coding: utf-8
-from autopath import do_autopath
+from .autopath import do_autopath
 do_autopath()
 
 from tiramisu.option import BoolOption, StrOption, SymLinkOption, \
@@ -11,6 +11,10 @@ from tiramisu.setting import groups, owners
 from py.test import raises
 
 
+def return_value():
+    pass
+
+
 #____________________________________________________________
 def test_symlink_option():
     boolopt = BoolOption("b", "", default=False)
@@ -31,6 +35,26 @@ def test_symlink_option():
     assert config.c is False
 
 
+def test_symlink_getproperties():
+    boolopt = BoolOption('b', '', default=True, properties=('test',))
+    linkopt = SymLinkOption("c", boolopt)
+    descr = OptionDescription('opt', '', [boolopt, linkopt])
+    config = Config(descr)
+    config.read_write()
+    assert boolopt.impl_getproperties() == linkopt.impl_getproperties() == ('test',)
+    assert boolopt.impl_has_callback() == linkopt.impl_has_callback() == False
+
+
+def test_symlink_getcallback():
+    boolopt = BoolOption('b', '', callback=return_value)
+    linkopt = SymLinkOption("c", boolopt)
+    descr = OptionDescription('opt', '', [boolopt, linkopt])
+    config = Config(descr)
+    config.read_write()
+    assert boolopt.impl_has_callback() == linkopt.impl_has_callback() == True
+    assert boolopt.impl_get_callback() == linkopt.impl_get_callback() == (return_value, {})
+
+
 def test_symlink_requires():
     boolopt = BoolOption('b', '', default=True)
     stropt = StrOption('s', '', requires=[{'option': boolopt,
