@@ -18,14 +18,15 @@
 # the whole pypy projet is under MIT licence
 # ____________________________________________________________
 "enables us to carry out a calculation and return an option's value"
-from .error import PropertiesOptionError, ConfigError, SlaveError
+import sys
+from .error import PropertiesOptionError, ConfigError, SlaveError, display_list
 from .i18n import _
 from .setting import undefined
 # ____________________________________________________________
 
 
 def carry_out_calculation(option, context, callback, callback_params,
-                          index=undefined, returns_raise=False):
+                          index=undefined, validate=True, is_validator=False):
     """a function that carries out a calculation for an option's value
 
     :param option: the option
@@ -38,6 +39,7 @@ def carry_out_calculation(option, context, callback, callback_params,
     :type callback_params: dict
     :param index: if an option is multi, only calculates the nth value
     :type index: int
+    :param is_validator: to know if carry_out_calculation is used to validate a value
 
     The callback_params is a dict. Key is used to build args (if key is '')
     and kwargs (otherwise). Values are tuple of:
@@ -148,7 +150,9 @@ def carry_out_calculation(option, context, callback, callback_params,
                     return undefined
                 if callbk[0] is None:  # pragma: optional cover
                     #Not an option, set full context
-                    tcparams.setdefault(key, []).append((context, False))
+                    tcparams.setdefault(key, []).append((context.duplicate(), False))
+                elif callbk[0] == 'index':
+                    tcparams.setdefault(key, []).append((index, False))
                 else:
                     # callbk is something link (opt, True|False)
                     opt, force_permissive = callbk
@@ -160,23 +164,29 @@ def carry_out_calculation(option, context, callback, callback_params,
                     else:
                         path = context.cfgimpl_get_description(
                         ).impl_get_path_by_opt(opt)
+                    # don't validate if option is option that we tried to validate
+                    if opt == option:
+                        valid = False
+                    else:
+                        valid = validate
                     # get value
                     value = context.getattr(path, force_permissive=True,
-                                            validate=False,
-                                            returns_raise=True)
+                                            validate=valid, returns_raise=True)
                     if isinstance(value, Exception):
                         if isinstance(value, PropertiesOptionError):
                             if force_permissive:
                                 continue
-                            err = ConfigError(_('unable to carry out a calculation'
-                                                ', option {0} has properties: {1} '
-                                                'for: {2}').format(opt.impl_getname(),
-                                                                   value.proptype,
-                                                                   option.impl_getname()))
-                            if returns_raise:
-                                return err
+                            if len(value.proptype) == 1:
+                                prop_name = _('the property')
                             else:
-                                raise err
+                                prop_name = _('the properties')
+                            err = ConfigError(_('unable to carry out a calculation for option "{2}" '
+                                                'because the option "{0}" has {3} {1}'
+                                                ).format(value._datas['opt'].impl_getname(),
+                                                         display_list(value.proptype),
+                                                         option.impl_getname(),
+                                                         prop_name))
+                            return err
                         else:
                             raise value
                     # convert to list, not modifie this multi
@@ -184,7 +194,7 @@ def carry_out_calculation(option, context, callback, callback_params,
                         has_option = True
                         value = list(value)
 
-                    if opt.impl_is_master_slaves() and \
+                    if opt != option and not option.impl_is_optiondescription() and opt.impl_is_master_slaves() and \
                             opt.impl_get_master_slaves().in_same_group(option):
                         master_slave = True
                         is_multi = True
@@ -213,7 +223,7 @@ def carry_out_calculation(option, context, callback, callback_params,
                     args.append(val)
                 else:
                     kwargs[key] = val
-        return calculate(callback, args, kwargs, returns_raise)
+        return calculate(option, callback, is_validator, args, kwargs)
     else:
         # no value is multi
         # return a single value
@@ -226,14 +236,13 @@ def carry_out_calculation(option, context, callback, callback_params,
                     args.append(couple[0])
                 else:
                     kwargs[key] = couple[0]
-        ret = calculate(callback, args, kwargs, returns_raise)
+        ret = calculate(option, callback, is_validator, args, kwargs)
         if not option.impl_is_optiondescription() and callback_params != {} and isinstance(ret, list) and \
                 option.impl_is_master_slaves('slave'):
             if not has_option and index not in [None, undefined]:
                 if index < len(ret):
                     ret = ret[index]
                 else:
-                    #FIXME really?
                     ret = None
             else:
                 raise SlaveError(_("callback cannot return a list for a "
@@ -241,7 +250,7 @@ def carry_out_calculation(option, context, callback, callback_params,
         return ret
 
 
-def calculate(callback, args, kwargs, returns_raise):
+def calculate(option, callback, is_validator, args, kwargs):
     """wrapper that launches the 'callback'
 
     :param callback: callback function
@@ -249,10 +258,29 @@ def calculate(callback, args, kwargs, returns_raise):
     :param kwargs: in the callback's arity, the named parameters
 
     """
-    if returns_raise:
-        try:
-            return callback(*args, **kwargs)
-        except ValueError as err:
-            return err
-    else:
+    try:
         return callback(*args, **kwargs)
+    except ValueError as err:
+        if is_validator:
+            return err
+        error = err
+    except Exception as err:
+        error = err
+
+    if sys.version_info[0] >= 3:  # pragma: optional cover
+        func_name = callback.__name__
+    else:
+        func_name = callback.func_name
+    if len(args) != 0 or len(kwargs) != 0:
+        msg = _('unexpected error "{0}" in function "{1}" with arguments "{3}" and "{4}" '
+                'for option "{2}"').format(str(error),
+                                           func_name,
+                                           option.impl_get_display_name(),
+                                           args,
+                                           kwargs)
+    else:
+        msg = _('unexpected error "{0}" in function "{1}" for option "{2}"'
+                '').format(str(error),
+                           func_name,
+                           option.impl_get_display_name())
+    raise ConfigError(msg)
