@@ -125,7 +125,7 @@ class SubConfig(object):
     def iter_all(self, force_permissive=False):
         """A way of parsing options **and** groups.
         iteration on Options and OptionDescriptions."""
-        for child in self.cfgimpl_get_description().impl_getchildren():
+        for child in self.cfgimpl_get_description()._impl_getchildren(context=self._cfgimpl_get_context()):
             try:
                 yield child.impl_getname(), self.getattr(child.impl_getname(),
                                                          force_permissive=force_permissive)
@@ -187,6 +187,9 @@ class SubConfig(object):
             raise ConfigError(_('the context does not exist anymore'))
         return context
 
+    def cfgimpl_get_context(self):
+        return self._cfgimpl_get_context()
+
     def cfgimpl_get_description(self):
         if self._impl_descr is None:  # pragma: optional cover
             raise ConfigError(_('no option description found for this config'
@@ -212,15 +215,14 @@ class SubConfig(object):
         self.setattr(name, value, force_permissive=force_permissive,
                      not_raises=not_raises)
 
-    def setattr(self, name, value, force_permissive=False, not_raises=False):
+    def setattr(self, name, value, force_permissive=False, not_raises=False, index=None):
         if name.startswith('_impl_'):
-            object.__setattr__(self, name, value)
-            return
+            return object.__setattr__(self, name, value)
         if '.' in name:  # pragma: optional cover
             homeconfig, name = self.cfgimpl_get_home_by_path(name,
                                                              force_permissive=force_permissive)
             return homeconfig.setattr(name, value, force_permissive,
-                                      not_raises)
+                                      not_raises, index=index)
         context = self._cfgimpl_get_context()
         child = self.cfgimpl_get_description().__getattr__(name,
                                                            context=context)
@@ -230,12 +232,12 @@ class SubConfig(object):
                 not isinstance(child, DynSymLinkOption):  # pragma: no dynoptiondescription cover
             path = context.cfgimpl_get_description().impl_get_path_by_opt(
                 child._impl_getopt())
-            context.setattr(path, value, force_permissive, not_raises)
+            context.setattr(path, value, force_permissive, not_raises, index=index)
         else:
             subpath = self._get_subpath(name)
             self.cfgimpl_get_values().setitem(child, value, subpath,
                                               force_permissive=force_permissive,
-                                              not_raises=not_raises)
+                                              not_raises=not_raises, index=index)
 
     def __delattr__(self, name):
         context = self._cfgimpl_get_context()
@@ -258,7 +260,7 @@ class SubConfig(object):
         return subpath
 
     def getattr(self, name, force_permissive=False, validate=True,
-                _setting_properties=undefined, index=None,
+                _setting_properties=undefined, _self_properties=undefined, index=None,
                 returns_raise=False):
         """
         attribute notation mechanism for accessing the value of an option
@@ -273,47 +275,57 @@ class SubConfig(object):
                 name, force_permissive=force_permissive,
                 returns_raise=returns_raise)
             if isinstance(homeconfig, Exception):
-                return homeconfig
-            return homeconfig.getattr(name, force_permissive=force_permissive,
-                                      validate=validate,
-                                      _setting_properties=_setting_properties,
-                                      index=index, returns_raise=returns_raise)
-        context = self._cfgimpl_get_context()
-        option = self.cfgimpl_get_description().__getattr__(name,
-                                                            context=context)
-        subpath = self._get_subpath(name)
-        if isinstance(option, DynSymLinkOption):
-            return self.cfgimpl_get_values()._get_cached_value(
-                option, path=subpath,
-                validate=validate,
-                force_permissive=force_permissive,
-                setting_properties=_setting_properties, index=index,
-                returns_raise=returns_raise)
-        elif isinstance(option, SymLinkOption):  # pragma: no dynoptiondescription cover
-            path = context.cfgimpl_get_description().impl_get_path_by_opt(
-                option._impl_getopt())
-            return context.getattr(path, validate=validate,
-                                   force_permissive=force_permissive,
-                                   _setting_properties=_setting_properties,
-                                   index=index, returns_raise=returns_raise)
-        elif option.impl_is_optiondescription():
-            props = self.cfgimpl_get_settings().validate_properties(
-                option, True, False, path=subpath,
-                force_permissive=force_permissive,
-                setting_properties=_setting_properties)
-            if props:
-                if returns_raise:
-                    return props
-                else:
-                    raise props
-            return SubConfig(option, self._impl_context, subpath)
+                cfg = homeconfig
+            else:
+                cfg = homeconfig.getattr(name, force_permissive=force_permissive,
+                                         validate=validate,
+                                         _setting_properties=_setting_properties,
+                                         _self_properties=_self_properties,
+                                         index=index, returns_raise=returns_raise)
         else:
-            return self.cfgimpl_get_values()._get_cached_value(
-                option, path=subpath,
-                validate=validate,
-                force_permissive=force_permissive,
-                setting_properties=_setting_properties,
-                index=index, returns_raise=returns_raise)
+            context = self._cfgimpl_get_context()
+            option = self.cfgimpl_get_description().__getattr__(name,
+                                                                context=context)
+            subpath = self._get_subpath(name)
+            if isinstance(option, DynSymLinkOption):
+                cfg = self.cfgimpl_get_values()._get_cached_value(
+                    option, path=subpath,
+                    validate=validate,
+                    force_permissive=force_permissive,
+                    setting_properties=_setting_properties,
+                    self_properties=_self_properties,
+                    index=index)
+            elif isinstance(option, SymLinkOption):  # pragma: no dynoptiondescription cover
+                path = context.cfgimpl_get_description().impl_get_path_by_opt(
+                    option._impl_getopt())
+                cfg = context.getattr(path, validate=validate,
+                                      force_permissive=force_permissive,
+                                      _setting_properties=_setting_properties,
+                                      _self_properties=_self_properties,
+                                      index=index, returns_raise=True)
+            elif option.impl_is_optiondescription():
+                props = self.cfgimpl_get_settings().validate_properties(
+                    option, True, False, path=subpath,
+                    force_permissive=force_permissive,
+                    self_properties=_self_properties,
+                    setting_properties=_setting_properties)
+                if props:
+                    if returns_raise:
+                        return props
+                    else:
+                        raise props
+                return SubConfig(option, self._impl_context, subpath)
+            else:
+                cfg = self.cfgimpl_get_values()._get_cached_value(
+                    option, path=subpath,
+                    validate=validate,
+                    force_permissive=force_permissive,
+                    setting_properties=_setting_properties,
+                    self_properties=_self_properties,
+                    index=index)
+        if not returns_raise and isinstance(cfg, Exception):
+            raise cfg
+        return cfg
 
     def find(self, bytype=None, byname=None, byvalue=undefined, type_='option',
              check_properties=True, force_permissive=False):
@@ -426,7 +438,7 @@ class SubConfig(object):
 
     def make_dict(self, flatten=False, _currpath=None, withoption=None,
                   withvalue=undefined, force_permissive=False,
-                  setting_properties=undefined):
+                  setting_properties=undefined, fullpath=False):
         """exports the whole config into a `dict`, for example:
 
         >>> print cfg.make_dict()
@@ -496,21 +508,23 @@ class SubConfig(object):
                         path = path[len(tmypath):]
                 self._make_sub_dict(opt, path, pathsvalues, _currpath, flatten,
                                     force_permissive=force_permissive,
-                                    setting_properties=setting_properties)
+                                    setting_properties=setting_properties,
+                                    fullpath=fullpath)
         #withoption can be set to None below !
         if withoption is None:
-            for opt in self.cfgimpl_get_description().impl_getchildren():
+            for opt in self.cfgimpl_get_description()._impl_getchildren(context=self._cfgimpl_get_context()):
                 path = opt.impl_getname()
                 self._make_sub_dict(opt, path, pathsvalues, _currpath, flatten,
                                     force_permissive=force_permissive,
-                                    setting_properties=setting_properties)
+                                    setting_properties=setting_properties,
+                                    fullpath=fullpath)
         if _currpath == []:
             options = dict(pathsvalues)
             return options
         return pathsvalues
 
     def _make_sub_dict(self, opt, path, pathsvalues, _currpath, flatten,
-                       setting_properties, force_permissive=False):
+                       setting_properties, force_permissive=False, fullpath=False):
         value = self.getattr(path,
                              force_permissive=force_permissive,
                              _setting_properties=setting_properties,
@@ -523,12 +537,20 @@ class SubConfig(object):
                     pathsvalues += value.make_dict(flatten,
                                                    _currpath + path.split('.'),
                                                    force_permissive=force_permissive,
-                                                   setting_properties=setting_properties)
+                                                   setting_properties=setting_properties,
+                                                   fullpath=fullpath)
             else:
                 if flatten:
                     name = opt.impl_getname()
                 else:
-                    name = '.'.join(_currpath + [opt.impl_getname()])
+                    if fullpath:
+                        root_path = self.cfgimpl_get_path()
+                        if root_path is None:
+                            name = opt.impl_getname()
+                        else:
+                            name = '.'.join([root_path, opt.impl_getname()])
+                    else:
+                        name = '.'.join(_currpath + [opt.impl_getname()])
                 pathsvalues.append((name, value))
 
     def cfgimpl_get_path(self, dyn=True):
@@ -655,10 +677,14 @@ class _CommonConfig(SubConfig):
         return fake_config
 
     def duplicate(self):
-        config = Config(self._impl_descr)
-        config.cfgimpl_get_values()._p_._values = self.cfgimpl_get_values()._p_._values
-        config.cfgimpl_get_settings()._p_._properties = self.cfgimpl_get_settings()._p_.get_modified_properties()
-        config.cfgimpl_get_settings()._p_._permissives = self.cfgimpl_get_settings()._p_.get_modified_permissives()
+        config = Config(self._impl_descr, _duplicate=True)
+        session = self.cfgimpl_get_values()._p_.getsession()
+        config.cfgimpl_get_values()._p_.importation(self.cfgimpl_get_values()._p_.exportation(
+            session))
+        config.cfgimpl_get_settings()._p_._properties = self.cfgimpl_get_settings(
+            )._p_.get_modified_properties()
+        config.cfgimpl_get_settings()._p_._permissives = self.cfgimpl_get_settings(
+            )._p_.get_modified_permissives()
         return config
 
 
@@ -668,7 +694,8 @@ class Config(_CommonConfig):
     __slots__ = ('__weakref__', '_impl_test', '_impl_name')
 
     def __init__(self, descr, session_id=None, persistent=False,
-                 name=undefined, force_values=None, force_settings=None):
+                 name=undefined, force_values=None, force_settings=None,
+                 _duplicate=False):
         """ Configuration option management master class
 
         :param descr: describes the configuration schema
@@ -698,7 +725,7 @@ class Config(_CommonConfig):
         self._impl_meta = None
         #undocumented option used only in test script
         self._impl_test = False
-        if force_settings is None or force_values is None:
+        if _duplicate is False and (force_settings is None or force_values is None):
             self._impl_build_all_caches()
         self._impl_name = name
 
@@ -826,13 +853,15 @@ class GroupConfig(_CommonConfig):
         return ret
 
     def getattr(self, name, force_permissive=False, validate=True,
-                _setting_properties=undefined,
+                _setting_properties=undefined, _self_properties=undefined, index=None,
                 returns_raise=False):
         for child in self._impl_children:
             if name == child._impl_name:
                 return child
         return super(GroupConfig, self).getattr(name, force_permissive,
                                                 validate,
+                                                _self_properties=_self_properties,
+                                                index=index,
                                                 _setting_properties=_setting_properties,
                                                 returns_raise=returns_raise)
 
