@@ -6,12 +6,12 @@ from .option import (ChoiceOption, BoolOption, IntOption, FloatOption,
                      StrOption, UnicodeOption, IPOption, PortOption,
                      NetworkOption, NetmaskOption, BroadcastOption,
                      DomainnameOption, EmailOption, URLOption, UsernameOption,
-                     FilenameOption, PasswordOption)
+                     DateOption, FilenameOption, PasswordOption)
 
 
 __all__ = ('MasterSlaves', 'OptionDescription', 'DynOptionDescription',
            'SynDynOptionDescription', 'Option', 'SymLinkOption',
-           'DynSymLinkOption', 'ChoiceOption', 'BoolOption',
+           'DynSymLinkOption', 'ChoiceOption', 'BoolOption', 'DateOption',
            'IntOption', 'FloatOption', 'StrOption', 'UnicodeOption',
            'IPOption', 'PortOption', 'NetworkOption', 'NetmaskOption',
            'BroadcastOption', 'DomainnameOption', 'EmailOption', 'URLOption',
