@@ -21,13 +21,14 @@
 # ____________________________________________________________
 import re
 import sys
+import datetime
 from IPy import IP
 from types import FunctionType
 from ..setting import undefined
 
 from ..error import ConfigError
 from ..i18n import _
-from .baseoption import Option, validate_callback
+from .baseoption import Option, validate_callback, display_list
 from ..autolib import carry_out_calculation
 
 
@@ -37,6 +38,7 @@ class ChoiceOption(Option):
     The option can also have the value ``None``
     """
     __slots__ = tuple()
+    _display_name = _('choice')
 
     def __init__(self, name, doc, values, default=None,
                  values_params=None, default_multi=None, requires=None,
@@ -51,12 +53,11 @@ class ChoiceOption(Option):
         else:
             if values_params is not None:
                 raise ValueError(_('values is not a function, so values_params must be None'))
-            if not isinstance(values, tuple):  # pragma: optional cover
+            if not isinstance(values, tuple):
                 raise TypeError(_('values must be a tuple or a function for {0}'
                                   ).format(name))
-        _setattr = object.__setattr__
-        _setattr(self, '_choice_values', values)
-        _setattr(self, '_choice_values_params', values_params)
+        session = self.getsession()
+        self.impl_set_choice_values_params(values, values_params, session)
         super(ChoiceOption, self).__init__(name, doc, default=default,
                                            default_multi=default_multi,
                                            callback=callback,
@@ -66,10 +67,11 @@ class ChoiceOption(Option):
                                            validator=validator,
                                            validator_params=validator_params,
                                            properties=properties,
-                                           warnings_only=warnings_only)
+                                           warnings_only=warnings_only,
+                                           session=session)
+        self.commit(session)
 
-    def impl_get_values(self, context, current_opt=undefined,
-                        returns_raise=False):
+    def impl_get_values(self, context, current_opt=undefined):
         if current_opt is undefined:
             current_opt = self
         #FIXME cache? but in context...
@@ -78,74 +80,71 @@ class ChoiceOption(Option):
             if context is None:
                 values = []
             else:
-                values_params = self._choice_values_params
-                if values_params is None:
-                    values_params = {}
                 values = carry_out_calculation(current_opt, context=context,
                                                callback=values,
-                                               callback_params=values_params,
-                                               returns_raise=returns_raise)
+                                               callback_params=self.impl_get_choice_values_params())
                 if isinstance(values, Exception):
                     return values
-                if values is not undefined and not isinstance(values, list):  # pragma: optional cover
+                if values is not undefined and not isinstance(values, list):
                     raise ConfigError(_('calculated values for {0} is not a list'
                                         '').format(self.impl_getname()))
         return values
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
-        values = self.impl_get_values(context, current_opt=current_opt,
-                                      returns_raise=returns_raise)
+
+    def _validate(self, value, context=undefined, current_opt=undefined):
+        values = self.impl_get_values(context, current_opt=current_opt)
         if isinstance(values, Exception):
             return values
-        if values is not undefined and not value in values:  # pragma: optional cover
-            return ValueError(_('value {0} is not permitted, '
-                                'only {1} is allowed'
-                                '').format(value,
-                                           values))
+        if values is not undefined and not value in values:
+            if len(values) == 1:
+                return ValueError(_('only {0} is allowed'
+                                    '').format(values[0]))
+            else:
+                return ValueError(_('only {0} are allowed'
+                                    '').format(display_list(values)))
 
 
 class BoolOption(Option):
     "represents a choice between ``True`` and ``False``"
     __slots__ = tuple()
+    _display_name = _('boolean')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         if not isinstance(value, bool):
-            return ValueError(_('invalid boolean'))  # pragma: optional cover
+            return ValueError()
 
 
 class IntOption(Option):
     "represents a choice of an integer"
     __slots__ = tuple()
+    _display_name = _('integer')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         if not isinstance(value, int):
-            return ValueError(_('invalid integer'))  # pragma: optional cover
+            return ValueError()
 
 
 class FloatOption(Option):
     "represents a choice of a floating point number"
     __slots__ = tuple()
+    _display_name = _('float')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         if not isinstance(value, float):
-            return ValueError(_('invalid float'))  # pragma: optional cover
+            return ValueError()
 
 
 class StrOption(Option):
     "represents the choice of a string"
     __slots__ = tuple()
+    _display_name = _('string')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         if not isinstance(value, str):
-            return ValueError(_('invalid string'))  # pragma: optional cover
+            return ValueError()
 
 
-if sys.version_info[0] >= 3:  # pragma: optional cover
+if sys.version_info[0] >= 3:  # pragma: no cover
     #UnicodeOption is same as StrOption in python 3+
     class UnicodeOption(StrOption):
         __slots__ = tuple()
@@ -155,19 +154,19 @@ else:
         "represents the choice of a unicode string"
         __slots__ = tuple()
         _empty = u''
+        _display_name = _('unicode string')
 
-        def _validate(self, value, context=undefined, current_opt=undefined,
-                      returns_raise=False):
+        def _validate(self, value, context=undefined, current_opt=undefined):
             if not isinstance(value, unicode):
-                return ValueError(_('invalid unicode'))  # pragma: optional cover
+                return ValueError()
 
 
 class PasswordOption(Option):
     "represents the choice of a password"
     __slots__ = tuple()
+    _display_name = _('password')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
@@ -176,6 +175,7 @@ class PasswordOption(Option):
 class IPOption(Option):
     "represents the choice of an ip"
     __slots__ = tuple()
+    _display_name = _('IP')
 
     def __init__(self, name, doc, default=None, default_multi=None,
                  requires=None, multi=False, callback=None,
@@ -196,56 +196,53 @@ class IPOption(Option):
                                        warnings_only=warnings_only,
                                        extra=extra)
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         # sometimes an ip term starts with a zero
         # but this does not fit in some case, for example bind does not like it
         err = self._impl_valid_unicode(value)
         if err:
             return err
         if value.count('.') != 3:
-            return ValueError(_('invalid IP'))
+            return ValueError()
         for val in value.split('.'):
             if val.startswith("0") and len(val) > 1:
-                return ValueError(_('invalid IP'))  # pragma: optional cover
+                return ValueError()
         # 'standard' validation
         try:
             IP('{0}/32'.format(value))
-        except ValueError:  # pragma: optional cover
-            return ValueError(_('invalid IP'))
+        except ValueError:
+            return ValueError()
 
     def _second_level_validation(self, value, warnings_only):
         ip = IP('{0}/32'.format(value))
-        if not self._get_extra('_allow_reserved') and ip.iptype() == 'RESERVED':  # pragma: optional cover
+        if not self._get_extra('_allow_reserved') and ip.iptype() == 'RESERVED':
             if warnings_only:
-                msg = _("IP is in reserved class")
+                msg = _("shouldn't in reserved class")
             else:
-                msg = _("invalid IP, mustn't be in reserved class")
+                msg = _("mustn't be in reserved class")
             return ValueError(msg)
-        if self._get_extra('_private_only') and not ip.iptype() == 'PRIVATE':  # pragma: optional cover
+        if self._get_extra('_private_only') and not ip.iptype() == 'PRIVATE':
             if warnings_only:
-                msg = _("IP is not in private class")
+                msg = _("should be in private class")
             else:
-                msg = _("invalid IP, must be in private class")
+                msg = _("must be in private class")
             return ValueError(msg)
 
-    def _cons_in_network(self, opts, vals, warnings_only):
+    def _cons_in_network(self, current_opt, opts, vals, warnings_only):
         if len(vals) != 3:
-            raise ConfigError(_('invalid len for vals'))  # pragma: optional cover
+            return ConfigError(_('invalid len for vals'))
         if None in vals:
             return
         ip, network, netmask = vals
-        if IP(ip) not in IP('{0}/{1}'.format(network, netmask)):  # pragma: optional cover
+        if IP(ip) not in IP('{0}/{1}'.format(network, netmask)):
             if warnings_only:
-                msg = _('IP {0} ({1}) not in network {2} ({3}) with netmask {4}'
-                        ' ({5})')
+                msg = _('should be in network {0}/{1} ({2}/{3})')
             else:
-                msg = _('invalid IP {0} ({1}) not in network {2} ({3}) with '
-                        'netmask {4} ({5})')
-            return ValueError(msg.format(ip, opts[0].impl_getname(), network,
-                              opts[1].impl_getname(), netmask, opts[2].impl_getname()))
+                msg = _('must be in network {0}/{1} ({2}/{3})')
+            return ValueError(msg.format(network, netmask,
+                              opts[1].impl_getname(), opts[2].impl_getname()))
         # test if ip is not network/broadcast IP
-        return opts[2]._cons_ip_netmask((opts[2], opts[0]), (netmask, ip), warnings_only)
+        return opts[2]._cons_ip_netmask(current_opt, (opts[2], opts[0]), (netmask, ip), warnings_only)
 
 
 class PortOption(Option):
@@ -260,6 +257,7 @@ class PortOption(Option):
     """
     __slots__ = tuple()
     port_re = re.compile(r"^[0-9]*$")
+    _display_name = _('port')
 
     def __init__(self, name, doc, default=None, default_multi=None,
                  requires=None, multi=False, callback=None,
@@ -283,13 +281,12 @@ class PortOption(Option):
             elif not allowed:
                 is_finally = True
             elif allowed and is_finally:
-                raise ValueError(_('inconsistency in allowed range'))  # pragma: optional cover
+                raise ValueError(_('inconsistency in allowed range'))
             if allowed:
                 extra['_max_value'] = ports_max[index]
 
         if extra['_max_value'] is None:
-            raise ValueError(_('max value is empty'))  # pragma: optional cover
-
+            raise ValueError(_('max value is empty'))
 
         super(PortOption, self).__init__(name, doc, default=default,
                                          default_multi=default_multi,
@@ -303,33 +300,31 @@ class PortOption(Option):
                                          warnings_only=warnings_only,
                                          extra=extra)
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         if isinstance(value, int):
-            if sys.version_info[0] >= 3:  # pragma: optional cover
+            if sys.version_info[0] >= 3:  # pragma: no cover
                 value = str(value)
             else:
                 value = unicode(value)
         err = self._impl_valid_unicode(value)
         if err:
             return err
-        if self._get_extra('_allow_range') and ":" in str(value):  # pragma: optional cover
+        if self._get_extra('_allow_range') and ":" in str(value):
             value = str(value).split(':')
             if len(value) != 2:
-                return ValueError(_('invalid port, range must have two values '
-                                  'only'))
+                return ValueError(_('range must have two values only'))
             if not value[0] < value[1]:
-                return ValueError(_('invalid port, first port in range must be'
+                return ValueError(_('first port in range must be'
                                   ' smaller than the second one'))
         else:
             value = [value]
 
         for val in value:
             if not self.port_re.search(val):
-                return ValueError(_('invalid port'))
+                return ValueError()
             val = int(val)
-            if not self._get_extra('_min_value') <= val <= self._get_extra('_max_value'):  # pragma: optional cover
-                return ValueError(_('invalid port, must be an integer between {0} '
+            if not self._get_extra('_min_value') <= val <= self._get_extra('_max_value'):
+                return ValueError(_('must be an integer between {0} '
                                     'and {1}').format(self._get_extra('_min_value'),
                                                       self._get_extra('_max_value')))
 
@@ -337,58 +332,58 @@ class PortOption(Option):
 class NetworkOption(Option):
     "represents the choice of a network"
     __slots__ = tuple()
+    _display_name = _('network address')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
         if value.count('.') != 3:
-            return ValueError(_('invalid network address'))
+            return ValueError()
         for val in value.split('.'):
             if val.startswith("0") and len(val) > 1:
-                return ValueError(_('invalid network address'))
+                return ValueError()
         try:
             IP(value)
-        except ValueError:  # pragma: optional cover
-            return ValueError(_('invalid network address'))
+        except ValueError:
+            return ValueError()
 
     def _second_level_validation(self, value, warnings_only):
         ip = IP(value)
-        if ip.iptype() == 'RESERVED':  # pragma: optional cover
+        if ip.iptype() == 'RESERVED':
             if warnings_only:
-                msg = _("network address is in reserved class")
+                msg = _("shouldn't be in reserved class")
             else:
-                msg = _("invalid network address, mustn't be in reserved class")
+                msg = _("mustn't be in reserved class")
             return ValueError(msg)
 
 
 class NetmaskOption(Option):
     "represents the choice of a netmask"
     __slots__ = tuple()
+    _display_name = _('netmask address')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
         if value.count('.') != 3:
-            return ValueError(_('invalid netmask address'))
+            return ValueError()
         for val in value.split('.'):
             if val.startswith("0") and len(val) > 1:
-                return ValueError(_('invalid netmask address'))
+                return ValueError()
         try:
             IP('0.0.0.0/{0}'.format(value))
-        except ValueError:  # pragma: optional cover
-            return ValueError(_('invalid netmask address'))
+        except ValueError:
+            return ValueError()
 
-    def _cons_network_netmask(self, opts, vals, warnings_only):
+    def _cons_network_netmask(self, current_opt, opts, vals, warnings_only):
         #opts must be (netmask, network) options
         if None in vals:
             return
         return self.__cons_netmask(opts, vals[0], vals[1], False, warnings_only)
 
-    def _cons_ip_netmask(self, opts, vals, warnings_only):
+    def _cons_ip_netmask(self, current_opt, opts, vals, warnings_only):
         #opts must be (netmask, ip) options
         if None in vals:
             return
@@ -397,7 +392,7 @@ class NetmaskOption(Option):
     def __cons_netmask(self, opts, val_netmask, val_ipnetwork, make_net,
                        warnings_only):
         if len(opts) != 2:
-            raise ConfigError(_('invalid len for opts'))  # pragma: optional cover
+            return ConfigError(_('invalid len for opts'))
         msg = None
         try:
             ip = IP('{0}/{1}'.format(val_ipnetwork, val_netmask),
@@ -406,44 +401,44 @@ class NetmaskOption(Option):
             if make_net and ip.prefixlen() != 32:
                 val_ip = IP(val_ipnetwork)
                 if ip.net() == val_ip:
-                    msg = _("invalid IP {0} ({1}) with netmask {2},"
-                            " this IP is a network")
+                    msg = _("this is a network with netmask {0} ({1})")
                 if ip.broadcast() == val_ip:
-                    msg = _("invalid IP {0} ({1}) with netmask {2},"
-                            " this IP is a broadcast")
+                    msg = _("this is a broadcast with netmask {0} ({1})")
 
-        except ValueError:  # pragma: optional cover
+        except ValueError:
             if not make_net:
-                msg = _('invalid network {0} ({1}) with netmask {2}')
-        if msg is not None:  # pragma: optional cover
-            return ValueError(msg.format(val_ipnetwork, opts[1].impl_getname(),
-                                         val_netmask))
+                msg = _('with netmask {0} ({1})')
+        if msg is not None:
+            return ValueError(msg.format(val_netmask, opts[1].impl_getname()))
 
 
 class BroadcastOption(Option):
     __slots__ = tuple()
+    _display_name = _('broadcast address')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
+        if value.count('.') != 3:
+            return ValueError()
+        for val in value.split('.'):
+            if val.startswith("0") and len(val) > 1:
+                return ValueError()
         try:
             IP('{0}/32'.format(value))
-        except ValueError:  # pragma: optional cover
-            return ValueError(_('invalid broadcast address'))
+        except ValueError:
+            return ValueError()
 
-    def _cons_broadcast(self, opts, vals, warnings_only):
+    def _cons_broadcast(self, current_opt, opts, vals, warnings_only):
         if len(vals) != 3:
-            raise ConfigError(_('invalid len for vals'))  # pragma: optional cover
+            return ConfigError(_('invalid len for vals'))
         if None in vals:
             return
         broadcast, network, netmask = vals
         if IP('{0}/{1}'.format(network, netmask)).broadcast() != IP(broadcast):
-            return ValueError(_('invalid broadcast {0} ({1}) with network {2} '
-                                '({3}) and netmask {4} ({5})').format(
-                                    broadcast, opts[0].impl_getname(), network,
-                                    opts[1].impl_getname(), netmask, opts[2].impl_getname()))  # pragma: optional cover
+            return ValueError(_('with network {0}/{1} ({2}/{3})').format(
+                network, netmask, opts[1].impl_getname(), opts[2].impl_getname()))
 
 
 class DomainnameOption(Option):
@@ -454,6 +449,7 @@ class DomainnameOption(Option):
     fqdn: with tld, not supported yet
     """
     __slots__ = tuple()
+    _display_name = _('domain name')
 
     def __init__(self, name, doc, default=None, default_multi=None,
                  requires=None, multi=False, callback=None,
@@ -461,15 +457,29 @@ class DomainnameOption(Option):
                  properties=None, allow_ip=False, type_='domainname',
                  warnings_only=False, allow_without_dot=False):
         if type_ not in ['netbios', 'hostname', 'domainname']:
-            raise ValueError(_('unknown type_ {0} for hostname').format(type_))  # pragma: optional cover
+            raise ValueError(_('unknown type_ {0} for hostname').format(type_))
         extra = {'_dom_type': type_}
         if allow_ip not in [True, False]:
-            raise ValueError(_('allow_ip must be a boolean'))  # pragma: optional cover
+            raise ValueError(_('allow_ip must be a boolean'))
         if allow_without_dot not in [True, False]:
-            raise ValueError(_('allow_without_dot must be a boolean'))  # pragma: optional cover
+            raise ValueError(_('allow_without_dot must be a boolean'))
         extra['_allow_ip'] = allow_ip
         extra['_allow_without_dot'] = allow_without_dot
-        extra['_domain_re'] = re.compile(r'^[a-z\d][a-z\d\-]*$')
+        # FIXME should be
+        # regexp = r'^((?!-)[a-z0-9-]{1,63}(?<!-)\.{0,1})$'
+        if type_ == 'domainname':
+            if allow_without_dot:
+                min_time = 0
+            else:
+                min_time = 1
+            regexp = r'((?!-)[a-z0-9-]{{{1},{0}}}\.){{{1},}}[a-z0-9-]{{1,{0}}}'.format(self._get_len(type_), min_time)
+        else:
+            regexp = r'((?!-)[a-z0-9-]{{1,{0}}})'.format(self._get_len(type_))
+        if allow_ip:
+            regexp = r'^(?:{0}|(?:(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){{3}}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)))$'.format(regexp)
+        else:
+            regexp = r'^{0}$'.format(regexp)
+        extra['_domain_re'] = re.compile(regexp)
         extra['_has_upper'] = re.compile('[A-Z]')
 
         super(DomainnameOption, self).__init__(name, doc, default=default,
@@ -484,20 +494,25 @@ class DomainnameOption(Option):
                                                warnings_only=warnings_only,
                                                extra=extra)
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _get_len(self, type_):
+        if type_ == 'netbios':
+            return 15
+        else:
+            return 63
+
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
 
         def _valid_length(val):
             if len(val) < 1:
-                return ValueError(_("invalid domainname's length (min 1)"))
+                return ValueError(_("invalid length (min 1)"))
             if len(val) > part_name_length:
-                return ValueError(_("invalid domainname's length (max {0})"
+                return ValueError(_("invalid length (max {0})"
                                     "").format(part_name_length))
 
-        if self._get_extra('_allow_ip') is True:  # pragma: optional cover
+        if self._get_extra('_allow_ip') is True:
             try:
                 IP('{0}/32'.format(value))
                 return
@@ -509,16 +524,13 @@ class DomainnameOption(Option):
             except ValueError:
                 pass
             else:
-                raise ValueError(_('invalid domainname, must not be an IP'))
-        if self._get_extra('_dom_type') == 'netbios':
-            part_name_length = 15
-        else:
-            part_name_length = 63
+                return ValueError(_('must not be an IP'))
+        part_name_length = self._get_len(self._get_extra('_dom_type'))
         if self._get_extra('_dom_type') == 'domainname':
             if not self._get_extra('_allow_without_dot') and not "." in value:
-                return ValueError(_("invalid domainname, must have dot"))
+                return ValueError(_("must have dot"))
             if len(value) > 255:
-                return ValueError(_("invalid domainname's length (max 255)"))
+                return ValueError(_("invalid length (max 255)"))
             for dom in value.split('.'):
                 err = _valid_length(dom)
                 if err:
@@ -527,65 +539,28 @@ class DomainnameOption(Option):
             return _valid_length(value)
 
     def _second_level_validation(self, value, warnings_only):
-        def _valid_char(val):
-            if self._get_extra('_has_upper').search(val):
-                return ValueError(_('some characters are uppercase'))
-            if not self._get_extra('_domain_re').search(val):
-                if warnings_only:
-                    return ValueError(_('some characters may cause problems'))
-                else:
-                    return ValueError(_('invalid domainname'))
-        #not for IP
-        if self._get_extra('_allow_ip') is True:
-            try:
-                IP('{0}/32'.format(value))
-                return
-            except ValueError:
-                pass
-        if self._get_extra('_dom_type') == 'domainname':
-            for dom in value.split('.'):
-                return _valid_char(dom)
-        else:
-            return _valid_char(value)
-
-
-class EmailOption(DomainnameOption):
-    __slots__ = tuple()
-    username_re = re.compile(r"^[\w!#$%&'*+\-/=?^`{|}~.]+$")
-
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
-        err = self._impl_valid_unicode(value)
-        if err:
-            return err
-        splitted = value.split('@', 1)
-        if len(splitted) == 1:
-            return ValueError(_('invalid email address, must contains one @'))
-        username, domain = value.split('@', 1)
-        if not self.username_re.search(username):
-            return ValueError(_('invalid username in email address'))  # pragma: optional cover
-        err = super(EmailOption, self)._validate(domain)
-        if err:
-            return err
-        return super(EmailOption, self)._second_level_validation(domain, False)
-
-    def _second_level_validation(self, value, warnings_only):
-        pass
+        if self._get_extra('_has_upper').search(value):
+            return ValueError(_('some characters are uppercase'))
+        if not self._get_extra('_domain_re').search(value):
+            if warnings_only:
+                return ValueError(_('some characters may cause problems'))
+            else:
+                return ValueError()
 
 
 class URLOption(DomainnameOption):
     __slots__ = tuple()
     proto_re = re.compile(r'(http|https)://')
     path_re = re.compile(r"^[A-Za-z0-9\-\._~:/\?#\[\]@!%\$&\'\(\)\*\+,;=]+$")
+    _display_name = _('URL')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
         match = self.proto_re.search(value)
-        if not match:  # pragma: optional cover
-            return ValueError(_('invalid url, must start with http:// or '
+        if not match:
+            return ValueError(_('must start with http:// or '
                                 'https://'))
         value = value[len(match.group(0)):]
         # get domain/files
@@ -603,8 +578,8 @@ class URLOption(DomainnameOption):
         else:
             domain, port = splitted
         if not 0 <= int(port) <= 65535:
-            return ValueError(_('invalid url, port must be an between 0 and '
-                                '65536'))  # pragma: optional cover
+            return ValueError(_('port must be an between 0 and '
+                                '65536'))
         # validate domainname
         err = super(URLOption, self)._validate(domain)
         if err:
@@ -614,36 +589,52 @@ class URLOption(DomainnameOption):
             return err
         # validate file
         if files is not None and files != '' and not self.path_re.search(files):
-            return ValueError(_('invalid url, must ends with a valid resource name'))  # pragma: optional cover
+            return ValueError(_('must ends with a valid resource name'))
 
     def _second_level_validation(self, value, warnings_only):
         pass
 
 
-class UsernameOption(Option):
+class _RegexpOption(Option):
+    __slots__ = tuple()
+    def _validate(self, value, context=undefined, current_opt=undefined):
+        err = self._impl_valid_unicode(value)
+        if err:
+            return err
+        match = self._regexp.search(value)
+        if not match:
+            return ValueError()
+
+
+class EmailOption(_RegexpOption):
+    __slots__ = tuple()
+    #https://www.w3.org/TR/html-markup/input.email.html#input.email.attrs.value.single.
+    _regexp = re.compile(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$")
+    _display_name = _('email address')
+
+
+class UsernameOption(_RegexpOption):
     __slots__ = tuple()
     #regexp build with 'man 8 adduser' informations
-    username_re = re.compile(r"^[a-z_][a-z0-9_-]{0,30}[$a-z0-9_-]{0,1}$")
-
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
-        err = self._impl_valid_unicode(value)
-        if err:
-            return err
-        match = self.username_re.search(value)
-        if not match:
-            return ValueError(_('invalid username'))  # pragma: optional cover
+    _regexp = re.compile(r"^[a-z_][a-z0-9_-]{0,30}[$a-z0-9_-]{0,1}$")
+    _display_name = _('username')
 
 
-class FilenameOption(Option):
+class FilenameOption(_RegexpOption):
     __slots__ = tuple()
-    path_re = re.compile(r"^[a-zA-Z0-9\-\._~/+]+$")
+    _regexp = re.compile(r"^[a-zA-Z0-9\-\._~/+]+$")
+    _display_name = _('file name')
 
-    def _validate(self, value, context=undefined, current_opt=undefined,
-                  returns_raise=False):
+
+class DateOption(Option):
+    __slots__ = tuple()
+    _display_name = _('date')
+
+    def _validate(self, value, context=undefined, current_opt=undefined):
         err = self._impl_valid_unicode(value)
         if err:
             return err
-        match = self.path_re.search(value)
-        if not match:
-            return ValueError(_('invalid filename'))  # pragma: optional cover
+        try:
+            datetime.datetime.strptime(value,"%Y-%m-%d")
+        except ValueError as err:
+            return ValueError()
