@@ -24,7 +24,7 @@ import re
 
 from ..i18n import _
 from ..setting import groups, undefined, owners  # , log
-from .baseoption import BaseOption, SymLinkOption
+from .baseoption import BaseOption, SymLinkOption, Option, allowed_const_list
 from . import MasterSlaves
 from ..error import ConfigError, ConflictError
 from ..storage import get_storages_option
@@ -127,26 +127,33 @@ class OptionDescription(BaseOption, StorageOptionDescription):
             else:
                 option._set_readonly(True)
                 is_multi = option.impl_is_multi()
-                if not isinstance(option, SymLinkOption) and 'force_store_value' in option.impl_getproperties():
-                    force_store_values.append((subpath, option))
+                if not isinstance(option, SymLinkOption):
+                    properties = option.impl_getproperties()
+                    if 'force_store_value' in properties:
+                        force_store_values.append((subpath, option))
+                    if 'force_default_on_freeze' in properties and \
+                            'frozen' not in properties and \
+                            option.impl_is_master_slaves('master'):
+                        raise ConfigError(_('a master ({0}) cannot have '
+                                            '"force_default_on_freeze" property without "frozen"').format(subpath))
                 for func, all_cons_opts, params in option._get_consistencies():
-                    option._valid_consistencies(all_cons_opts[1:])
-                    if is_multi:
+                    option._valid_consistencies(all_cons_opts[1:], init=False)
+                    if func not in allowed_const_list and is_multi:
                         is_slave = option.impl_is_master_slaves()
                         if not is_slave:
-                            raise ValueError(_('malformed consistency option {0} '
+                            raise ValueError(_('malformed consistency option "{0}" '
                                                'must be a master/slaves').format(
                                                    option.impl_getname()))
                         masterslaves = option.impl_get_master_slaves()
                     for opt in all_cons_opts:
-                        if is_multi:
+                        if func not in allowed_const_list and is_multi:
                             if not opt.impl_is_master_slaves():
-                                raise ValueError(_('malformed consistency option {0} '
-                                                   'must not be a multi for {1}').format(
+                                raise ValueError(_('malformed consistency option "{0}" '
+                                                   'must not be a multi for "{1}"').format(
                                                        option.impl_getname(), opt.impl_getname()))
                             elif masterslaves != opt.impl_get_master_slaves():
-                                raise ValueError(_('malformed consistency option {0} '
-                                                   'must be in same master/slaves for {1}').format(
+                                raise ValueError(_('malformed consistency option "{0}" '
+                                                   'must be in same master/slaves for "{1}"').format(
                                                        option.impl_getname(), opt.impl_getname()))
                         _consistencies.setdefault(opt,
                                                   []).append((func,
@@ -229,7 +236,22 @@ class OptionDescription(BaseOption, StorageOptionDescription):
         if isinstance(group_type, groups.GroupType):
             self._group_type = group_type
             if isinstance(group_type, groups.MasterGroupType):
-                MasterSlaves(self.impl_getname(), self.impl_getchildren())
+                children = self.impl_getchildren()
+                for child in children:
+                    if isinstance(child, SymLinkOption):  # pragma: optional cover
+                        raise ValueError(_("master group {0} shall not have "
+                                           "a symlinkoption").format(self.impl_getname()))
+                    if not isinstance(child, Option):  # pragma: optional cover
+                        raise ValueError(_("master group {0} shall not have "
+                                           "a subgroup").format(self.impl_getname()))
+                    if not child.impl_is_multi():  # pragma: optional cover
+                        raise ValueError(_("not allowed option {0} "
+                                           "in group {1}"
+                                           ": this option is not a multi"
+                                           "").format(child.impl_getname(), self.impl_getname()))
+                #length of master change slaves length
+                self._set_has_dependency()
+                MasterSlaves(self.impl_getname(), children)
         else:  # pragma: optional cover
             raise ValueError(_('group_type: {0}'
                                ' not allowed').format(group_type))
@@ -292,8 +314,10 @@ class OptionDescription(BaseOption, StorageOptionDescription):
         values = carry_out_calculation(self, context=context,
                                        callback=callback,
                                        callback_params=callback_params)
+        if isinstance(values, Exception):
+            raise values
         if len(values) > len(set(values)):
-            raise ConfigError(_('DynOptionDescription callback return not uniq value'))
+            raise ConfigError(_('DynOptionDescription callback return not unique value'))
         for val in values:
             if not isinstance(val, str) or re.match(name_regexp, val) is None:
                 raise ValueError(_("invalid suffix: {0} for option").format(val))
@@ -365,7 +389,6 @@ class DynOptionDescription(OptionDescription):
                                     'dynoptiondescription'))
             child._impl_setsubdyn(self)
         self.impl_set_callback(callback, callback_params)
-        self.commit()
 
     def _validate_callback(self, callback, callback_params):
         if callback is None:
