@@ -20,7 +20,7 @@ from copy import copy
 from logging import getLogger
 import weakref
 from .error import (RequirementError, PropertiesOptionError,
-                    ConstError, ConfigError)
+                    ConstError, ConfigError, display_list)
 from .i18n import _
 
 
@@ -113,6 +113,7 @@ log = getLogger('tiramisu')
 #FIXME
 #import logging
 #logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)
+debug = False
 
 
 # ____________________________________________________________
@@ -277,6 +278,11 @@ class Property(object):
         :type propname: string
         """
         if propname in self._properties:
+            if propname == 'frozen' and 'force_default_on_freeze' in self._properties and \
+                    self._opt.impl_is_master_slaves('master'):
+                raise ConfigError(_('a master ({0}) cannot have '
+                                    '"force_default_on_freeze" property without "frozen"'
+                                    '').format(self._opt.impl_get_display_name()))
             self._properties.remove(propname)
             self._setting._setproperties(self._properties, self._path)
 
@@ -302,6 +308,9 @@ class Property(object):
     def __repr__(self):
         return str(list(self._properties))
 
+    def get(self):
+        return tuple(self._properties)
+
 
 #____________________________________________________________
 class Settings(object):
@@ -394,7 +403,7 @@ class Settings(object):
                 if opt.impl_is_multi() and not opt.impl_is_master_slaves('slave'):
                     props.add('empty')
                 if apply_requires:
-                    requires = self.apply_requires(opt, path, setting_properties, index)
+                    requires = self.apply_requires(opt, path, setting_properties, index, False)
                     if requires != set([]):
                         props = copy(props)
                         props |= requires
@@ -441,7 +450,7 @@ class Settings(object):
                             value=None, force_permissive=False,
                             setting_properties=undefined,
                             self_properties=undefined,
-                            index=None):
+                            index=None, debug=False):
         """
         validation upon the properties related to `opt_or_descr`
 
@@ -484,7 +493,7 @@ class Settings(object):
             elif 'empty' in properties and \
                     'empty' in setting_properties and \
                     self._getcontext().cfgimpl_get_values()._isempty(
-                        opt_or_descr, value, force_allow_empty_list=True):
+                        opt_or_descr, value, force_allow_empty_list=True, index=index):
                 properties.add('mandatory')
             # should return 'frozen' only when tried to modify a value
             if check_frozen and 'everything_frozen' in setting_properties:
@@ -496,22 +505,31 @@ class Settings(object):
         # at this point an option should not remain in properties
         if properties != frozenset():
             props = list(properties)
+            datas = {'opt': opt_or_descr, 'path': path, 'setting_properties': setting_properties,
+                     'index': index, 'debug': True}
+            if is_descr:
+                opt_type = 'optiondescription'
+            else:
+                opt_type = 'option'
             if 'frozen' in properties:
                 return PropertiesOptionError(_('cannot change the value for '
-                                               'option {0} this option is'
+                                               'option "{0}" this option is'
                                                ' frozen').format(
                                                    opt_or_descr.impl_getname()),
-                                             props)
+                                             props, self, datas, opt_type)
             else:
-                if is_descr:
-                    opt_type = 'optiondescription'
+                if len(props) == 1:
+                    prop_msg = _('property')
                 else:
-                    opt_type = 'option'
-                return PropertiesOptionError(_("trying to access to an {0} "
-                                               "named: {1} with properties {2}"
-                                               "").format(opt_type,
-                                                          opt_or_descr._name,
-                                                          str(props)), props)
+                    prop_msg = _('properties')
+                return PropertiesOptionError(_('cannot access to {0} "{1}" '
+                                               'because has {2} {3}'
+                                               '').format(opt_type,
+                                                          opt_or_descr.impl_get_display_name(),
+                                                          prop_msg,
+                                                          display_list(props)),
+                                               props,
+                                               self, datas, opt_type)
 
     def setpermissive(self, permissive, opt=None, path=None):
         """
@@ -528,6 +546,7 @@ class Settings(object):
         if not isinstance(permissive, tuple):  # pragma: optional cover
             raise TypeError(_('permissive must be a tuple'))
         self._p_.setpermissive(path, permissive)
+        self._getcontext().cfgimpl_reset_cache()
 
     #____________________________________________________________
     def setowner(self, owner):
@@ -571,7 +590,7 @@ class Settings(object):
         else:
             self._p_.reset_all_cache()
 
-    def apply_requires(self, opt, path, setting_properties, index):
+    def apply_requires(self, opt, path, setting_properties, index, debug):
         """carries out the jit (just in time) requirements between options
 
         a requirement is a tuple of this form that comes from the option's
@@ -608,20 +627,27 @@ class Settings(object):
           exception :exc:`~error.RequirementError` is raised.
 
         And at last, if no target option matches the expected values, the
-        action must be removed from the option's properties list.
+        action will not add to the option's properties list.
 
         :param opt: the option on wich the requirement occurs
         :type opt: `option.Option()`
         :param path: the option's path in the config
         :type path: str
         """
-        if opt.impl_getrequires() == []:
-            return frozenset()
+        current_requires = opt.impl_getrequires()
 
         # filters the callbacks
-        calc_properties = set()
+        if debug:
+            calc_properties = {}
+        else:
+            calc_properties = set()
+
+        if not current_requires:
+            return calc_properties
+
         context = self._getcontext()
-        for requires in opt.impl_getrequires():
+        all_properties = None
+        for requires in current_requires:
             for require in requires:
                 option, expected, action, inverse, \
                     transitive, same_action = require
@@ -632,6 +658,8 @@ class Settings(object):
                                              " '{0}' with requirement on: "
                                              "'{1}'").format(path, reqpath))
                 if option.impl_is_multi():
+                    if index is None:
+                        continue
                     idx = index
                 else:
                     idx = None
@@ -641,26 +669,48 @@ class Settings(object):
                 if isinstance(value, Exception):
                     if isinstance(value, PropertiesOptionError):
                         if not transitive:
-                            continue
+                            if all_properties is None:
+                                all_properties = []
+                                for requires in opt.impl_getrequires():
+                                    for require in requires:
+                                        all_properties.append(require[2])
+                            if not set(value.proptype) - set(all_properties):
+                                continue
                         properties = value.proptype
                         if same_action and action not in properties:  # pragma: optional cover
-                            raise RequirementError(_("option '{0}' has "
-                                                     "requirement's property "
-                                                     "error: "
-                                                     "{1} {2}").format(opt._name,
-                                                                       reqpath,
-                                                                       properties))
+                            if len(properties) == 1:
+                                prop_msg = _('property')
+                            else:
+                                prop_msg = _('properties')
+                            raise RequirementError(_('cannot access to option "{0}" because '
+                                                     'required option "{1}" has {2} {3}'
+                                                     '').format(opt.impl_get_display_name(),
+                                                                option.impl_get_display_name(),
+                                                                prop_msg,
+                                                                display_list(properties)))
+                        orig_value = value
                         # transitive action, force expected
                         value = expected[0]
                         inverse = False
                     else:
                         raise value
-                if (not inverse and
-                        value in expected or
+                else:
+                    orig_value = value
+                if (not inverse and value in expected or
                         inverse and value not in expected):
-                    calc_properties.add(action)
-                    # the calculation cannot be carried out
-                    break
+                    if debug:
+                        if isinstance(orig_value, PropertiesOptionError):
+                            for act, msg in orig_value._settings.apply_requires(**orig_value._datas).items():
+                                calc_properties.setdefault(action, []).extend(msg)
+                        else:
+                            if not inverse:
+                                msg = _('the value of "{0}" is "{1}"')
+                            else:
+                                msg = _('the value of "{0}" is not "{1}"')
+                            calc_properties.setdefault(action, []).append(msg.format(option.impl_get_display_name(), display_list(expected, 'or')))
+                    else:
+                        calc_properties.add(action)
+                        break
         return calc_properties
 
     def get_modified_properties(self):
