@@ -135,6 +135,8 @@ def get_storages_option(type_):
     imp = storage_option_type.get()
     if type_ == 'base':
         return imp.StorageBase
+    elif type_ == 'masterslaves':
+        return imp.StorageMasterSlaves
     else:
         return imp.StorageOptionDescription
 
