@@ -25,7 +25,7 @@ use it. But if something goes wrong, you will lost your modifications.
 from .value import Values
 from .setting import Settings
 from .storage import setting, Storage, list_sessions, delete_session
-from .option import StorageBase, StorageOptionDescription
+from .option import StorageBase, StorageOptionDescription, StorageMasterSlaves
 
 __all__ = (setting, Values, Settings, Storage, list_sessions, delete_session,
-           StorageBase, StorageOptionDescription)
+           StorageBase, StorageOptionDescription, StorageMasterSlaves)
