@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ""
-# Copyright (C) 2014 Team tiramisu (see AUTHORS for all contributors)
+# Copyright (C) 2014-2017 Team tiramisu (see AUTHORS for all contributors)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ from ...setting import undefined
 from ...error import ConfigError
 static_tuple = tuple()
 static_set = frozenset()
-if sys.version_info[0] >= 3:  # pragma: optional cover
+if sys.version_info[0] >= 3:  # pragma: no cover
     xrange = range
 
 
@@ -34,10 +34,12 @@ if sys.version_info[0] >= 3:  # pragma: optional cover
 class StorageBase(object):
     __slots__ = ('_name',
                  '_informations',
-                 '_multi',
                  '_extra',
                  '_warnings_only',
                  '_allow_empty_list',
+                 #multi
+                 '_multi',
+                 '_unique',
                  #value
                  '_default',
                  '_default_multi',
@@ -53,6 +55,7 @@ class StorageBase(object):
                  '_choice_values',
                  '_choice_values_params',
                  #other
+                 '_has_dependency',
                  '_state_master_slaves',
                  '_state_val_call',
                  '_state_requires',
@@ -65,7 +68,8 @@ class StorageBase(object):
                  )
 
     def __init__(self, name, multi, warnings_only, doc, extra, calc_properties,
-                 requires, properties, allow_empty_list, opt=undefined):
+                 requires, properties, allow_empty_list, unique, opt=undefined,
+                 session=None):
         _setattr = object.__setattr__
         _setattr(self, '_name', name)
         if doc is not undefined:
@@ -87,6 +91,8 @@ class StorageBase(object):
             _setattr(self, '_opt', opt)
         if allow_empty_list is not undefined:
             _setattr(self, '_allow_empty_list', allow_empty_list)
+        if unique is not undefined:
+            setattr(self, '_unique', unique)
 
     def _set_default_values(self, default, default_multi, is_multi):
         _setattr = object.__setattr__
@@ -97,7 +103,7 @@ class StorageBase(object):
             _setattr(self, '_default', default)
 
         if is_multi and default_multi is not None:
-            err = self._validate(default_multi, returns_raise=True)
+            err = self._validate(default_multi)
             if err:
                 raise ValueError(_("invalid default_multi value {0} "
                                    "for option {1}: {2}").format(
@@ -164,6 +170,12 @@ class StorageBase(object):
             val_call = tuple([callback, callback_params])
         self._val_call = (val, val_call)
 
+    def impl_set_choice_values_params(self, values, values_params, session):
+        self._choice_values = values
+        if values_params is not None:
+            self._choice_values_params = values_params
+
+
     def impl_get_callback(self):
         call = getattr(self, '_val_call', (None, None))[1]
         if call is None:
@@ -174,6 +186,9 @@ class StorageBase(object):
             ret_call = call
         return ret_call
 
+    def impl_get_choice_values_params(self):
+        return getattr(self, '_choice_values_params', {})
+
     def impl_get_calc_properties(self):
         return getattr(self, '_calc_properties', static_set)
 
@@ -259,6 +274,9 @@ class StorageBase(object):
                 if extra is not None:
                     _setattr(self, '_extra', tuple([tuple(extra.keys()), tuple(extra.values())]))
 
+    def impl_getproperties(self):
+        return self._properties
+
     def _impl_setsubdyn(self, subdyn):
         self._subdyn = subdyn
 
@@ -266,7 +284,7 @@ class StorageBase(object):
         self._opt = opt
 
     def _is_string(self, infos):
-        if sys.version_info[0] >= 3:  # pragma: optional cover
+        if sys.version_info[0] >= 3:  # pragma: no cover
             return isinstance(infos, str)
         else:
             return isinstance(infos, str) or isinstance(infos, unicode)
@@ -329,6 +347,9 @@ class StorageBase(object):
     def impl_allow_empty_list(self):
         return getattr(self, '_allow_empty_list', undefined)
 
+    def impl_is_unique(self):
+        return getattr(self, '_unique', False)
+
     def _get_extra(self, key):
         extra = self._extra
         if isinstance(extra, tuple):
@@ -357,7 +378,16 @@ class StorageBase(object):
         "accessing the default value for a multi"
         return getattr(self, '_default_multi', None)
 
-    def commit(self):
+    def _get_master_slave(self):
+        return getattr(self, '_master_slaves', None)
+
+    def _set_master_slaves(self, option):
+        self._master_slaves = option
+
+    def getsession(self):
+        pass
+
+    def commit(self, session):
         pass
 
 
@@ -392,13 +422,13 @@ class StorageOptionDescription(StorageBase):
             raise AttributeError(_('no option {0} found').format(opt))
         return self._cache_paths[1][self._cache_paths[0].index(opt)]
 
-    def impl_get_group_type(self):  # pragma: optional cover
+    def impl_get_group_type(self):
         return self._group_type
 
     def impl_build_cache_option(self, _currpath=None, cache_path=None,
                                 cache_option=None):
 
-        if _currpath is None and getattr(self, '_cache_paths', None) is not None:
+        if self.impl_is_readonly() or (_currpath is None and getattr(self, '_cache_paths', None) is not None):
             # cache already set
             return
         if _currpath is None:
@@ -435,7 +465,7 @@ class StorageOptionDescription(StorageBase):
                 if dynopt == subopt:
                     found = True
                     break
-            if not found:
+            if not found:  # pragma: no cover
                 raise ConfigError(_('cannot find dynpath'))
             subpath = subpath + suffix
             for slength in xrange(length, len(spath)):
@@ -491,7 +521,7 @@ class StorageOptionDescription(StorageBase):
                 return _filter_by_name(path, option)
 
         opts, paths = self._cache_paths
-        for index in range(0, len(paths)):
+        for index in xrange(0, len(paths)):
             option = opts[index]
             if option.impl_is_optiondescription():
                 continue
@@ -524,7 +554,7 @@ class StorageOptionDescription(StorageBase):
     def _getattr(self, name, suffix=undefined, context=undefined, dyn=True):
         error = False
         if suffix is not undefined:
-            if undefined in [suffix, context]:  # pragma: optional cover
+            if undefined in [suffix, context]:  # pragma: no cover
                 raise ConfigError(_("suffix and context needed if "
                                     "it's a dyn option"))
             if name.endswith(suffix):
@@ -549,3 +579,17 @@ class StorageOptionDescription(StorageBase):
             raise AttributeError(_('unknown Option {0} '
                                    'in OptionDescription {1}'
                                    '').format(name, self.impl_getname()))
+
+
+class StorageMasterSlaves(object):
+    __slots__ = ('master', 'slaves')
+
+    def __init__(self, master, slaves):
+        self.master = master
+        self.slaves = slaves
+
+    def _sm_getmaster(self):
+        return self.master
+
+    def _sm_getslaves(self):
+        return tuple(self.slaves)
