@@ -35,9 +35,6 @@ class Values(Cache):
     def getsession(self):
         pass
 
-    def delsession(self, session):
-        pass
-
     def _setvalue_info(self, nb, idx, value, values, index, vidx):
         lst = list(self._values[nb])
         if idx is None:
@@ -65,6 +62,7 @@ class Values(Cache):
                 lst[idx] = tuple(lst[idx])
         values.append(tuple(lst))
         return vidx
+
     # value
     def setvalue(self, path, value, owner, index, session):
         """set value for a path
@@ -82,8 +80,6 @@ class Values(Cache):
         if isinstance(value, list):
             value = tuple(value)
         vidx = self._setvalue_info(2, idx, value, values, index, vidx)
-        if isinstance(value, list):
-            value = tuple(value)
         self._setvalue_info(3, idx, owner, values, index, vidx)
         self._values = tuple(values)
 
@@ -99,6 +95,51 @@ class Values(Cache):
         """
         return path in self._values[0]
 
+    def reduce_index(self, path, index):
+        """
+        _values == ((path1, path2), ((idx1_1, idx1_2), None), ((value1_1, value1_2), value2), ((owner1_1, owner1_2), owner2))
+        """
+        path_idx = self._values[0].index(path)
+        indexes = self._values[1][path_idx]
+        if index in indexes:
+            subidx = indexes.index(index)
+            values = list(self._values)
+            values_idx = list(values[1])
+            lvalues = list(values_idx[path_idx])
+            lvalues[subidx] = lvalues[subidx] - 1
+            values_idx[path_idx] = tuple(lvalues)
+            values[1] = tuple(values_idx)
+            self._values = tuple(values)
+
+    def resetvalue_index(self, path, index):
+        def _resetvalue(nb):
+            values_idx = list(values[nb])
+            del(values_idx[path_idx])
+            values[nb] = tuple(values_idx)
+
+        def _resetvalue_index(nb):
+            values_idx = list(values[nb])
+            lvalues = list(values_idx[path_idx])
+            del(lvalues[subidx])
+            values_idx[path_idx] = tuple(lvalues)
+            values[nb] = tuple(values_idx)
+
+        path_idx = self._values[0].index(path)
+        indexes = self._values[1][path_idx]
+        if index in indexes:
+            subidx = indexes.index(index)
+            values = list(self._values)
+            if len(values[1][path_idx]) == 1:
+                _resetvalue(0)
+                _resetvalue(1)
+                _resetvalue(2)
+                _resetvalue(3)
+            else:
+                _resetvalue_index(1)
+                _resetvalue_index(2)
+                _resetvalue_index(3)
+            self._values = tuple(values)
+
     def resetvalue(self, path, session):
         """remove value means delete value in storage
         """
@@ -183,32 +224,20 @@ class Values(Cache):
         _values == ((path1, path2), ((idx1_1, idx1_2), None), ((value1_1, value1_2), value2), ((owner1_1, owner1_2), owner2))
         """
         if path in self._values[0]:
-            idx = self._values[0].index(path)
-            if isinstance(self._values[1][idx], tuple):
-                if index is None:
-                    raise ValueError('index is mandatory')
-            elif index is not None:
-                raise ValueError('index is forbidden')
-
-            if self._values[1][idx] is None:
-                if index is None:
-                    value = self._values[nb][idx]
-                else:
-                    value = self._values[nb][idx][index]
+            path_idx = self._values[0].index(path)
+            indexes = self._values[1][path_idx]
+            if indexes is None:
+                if index is not None:  # pragma: no cover
+                    raise ValueError('index is forbidden')
+                value = self._values[nb][path_idx]
             else:
-                if index is not None:
-                    if index in self._values[1][idx]:
-                        subidx = self._values[1][idx].index(index)
-                        value = self._values[nb][idx][subidx]
-                    else:
-                        value = undefined
+                if index is None:  # pragma: no cover
+                    raise ValueError('index is mandatory')
+                if index in indexes:
+                    subidx = indexes.index(index)
+                    value = self._values[nb][path_idx][subidx]
                 else:
-                    value = []
-                    for i in xrange(0, max(self._values[1][idx])):
-                        if i in self._values[1][idx]:
-                            value.append(self._values[nb][idx][self._values[1][idx].index(i)])
-                        else:
-                            value.append(undefined)
+                    value = undefined
         else:
             value = undefined
         if isinstance(value, tuple):
