@@ -40,8 +40,9 @@ def load_requires(collection_type, proxy):
             return None
         ret = []
         requires = getattr(obj, proxy.value_attr)
+        session = util.Session()
         for require in requires:
-            option = util.session.query(_Base).filter_by(id=require.option).first()
+            option = session.query(_Base).filter_by(id=require.option).first()
             ret.append(tuple([option, require.expected, require.action, require.inverse, require.transitive, require.same_action]))
         return tuple(ret)
 
@@ -128,13 +129,14 @@ class _PropertyOption(SqlAlchemyBase):
 class _Information(SqlAlchemyBase):
     __tablename__ = 'information'
     id = Column(Integer, primary_key=True)
-    option = Column(Integer, ForeignKey('baseoption.id'), nullable=False)
+    option = Column(String, ForeignKey('baseoption.id'), nullable=False)
     key = Column(String)
     value = Column(PickleType)
 
-    def __init__(self, key, value):
-        self.key = key
-        self.value = value
+#    def __init__(self, option, key, value):
+#        self.option = option
+#        self.key = key
+#        self.value = value
 
 
 #____________________________________________________________
@@ -146,11 +148,12 @@ def load_callback_parm(collection_type, proxy):
             return None
         ret = []
         requires = getattr(obj, proxy.value_attr)
+        session = util.Session()
         for require in requires:
             if require.value is not None:
                 ret.append(require.value)
             else:
-                option = util.session.query(_Base).filter_by(id=require.option).first()
+                option = session.query(_Base).filter_by(id=require.option).first()
                 ret.append((option, require.force_permissive))
         return tuple(ret)
 
@@ -195,6 +198,152 @@ class _CallbackParam(SqlAlchemyBase):
                 self.params.append(_CallbackParamOption(value=param))
 
 
+#____________________________________________________________
+#
+# choice
+class _ChoiceParamOption(SqlAlchemyBase):
+    __tablename__ = 'choice_param_option'
+    id = Column(Integer, primary_key=True)
+    choice = Column(Integer, index=True)
+    option = Column(Integer)
+    force_permissive = Column(Boolean)
+    value = Column(PickleType)
+
+    def __init__(self, choice, option=undefined, force_permissive=undefined,  value=undefined):
+        self.choice = choice.id
+        if value is not undefined:
+            self.value = value
+        elif option is not undefined:
+            self.option = option.id
+            self.force_permissive = force_permissive
+
+
+class _ChoiceParam(SqlAlchemyBase):
+    __tablename__ = 'choice_param'
+    id = Column(Integer, primary_key=True)
+    option = Column(Integer, index=True)
+    key = Column(String)
+
+    def __init__(self, option, key):
+        self.option = option.id
+        self.key = key
+
+
+#def load_choice_parm(collection_type, proxy):
+#    def getter(obj):
+#        if obj is None:
+#            return None
+#        ret = []
+#        requires = getattr(obj, proxy.value_attr)
+#        session = util.Session()
+#        for require in requires:
+#            if require.value is not None:
+#                ret.append(require.value)
+#            else:
+#                option = session.query(_Base).filter_by(id=require.option).first()
+#                ret.append((option, require.force_permissive))
+#        return tuple(ret)
+#
+#    def setter(obj, value):
+#        setattr(obj, proxy.value_attr, value)
+#    return getter, setter
+#
+#
+#class _ChoiceParamOption(SqlAlchemyBase):
+#    __tablename__ = 'choice_param_option'
+#    id = Column(Integer, primary_key=True)
+#    valid_param = Column(Integer, ForeignKey('choice_param.id'))
+#    option = Column(Integer)
+#    force_permissive = Column(Boolean)
+#    value = Column(PickleType)
+#
+#    def __init__(self, option=undefined, force_permissive=undefined,  value=undefined):
+#        if value is not undefined:
+#            self.value = value
+#        elif option is not undefined:
+#            self.option = option.id
+#            self.force_permissive = force_permissive
+#
+#
+#class _ChoiceParam(SqlAlchemyBase):
+#    __tablename__ = 'choice_param'
+#    id = Column(Integer, primary_key=True)
+#    choice = Column(Integer, ForeignKey('baseoption.id'))
+#    key = Column(String)
+#    params = relationship('_ChoiceParamOption')
+#
+#    def __init__(self, key, params):
+#        self.key = key
+#        for param in params:
+#            if isinstance(param, tuple):
+#                if param == (None,):
+#                    self.params.append(_ChoiceParamOption())
+#                else:
+#                    self.params.append(_ChoiceParamOption(option=param[0],
+#                                                            force_permissive=param[1]))
+#            else:
+#                self.params.append(_ChoiceParamOption(value=param))
+
+
+#____________________________________________________________
+#
+# validator
+def load_validator_parm(collection_type, proxy):
+    def getter(obj):
+        if obj is None:
+            return None
+        ret = []
+        requires = getattr(obj, proxy.value_attr)
+        session = util.Session()
+        for require in requires:
+            if require.value is not None:
+                ret.append(require.value)
+            else:
+                option = session.query(_Base).filter_by(id=require.option).first()
+                ret.append((option, require.force_permissive))
+        return tuple(ret)
+
+    def setter(obj, value):
+        setattr(obj, proxy.value_attr, value)
+    return getter, setter
+
+
+class _ValidatorParamOption(SqlAlchemyBase):
+    __tablename__ = 'validator_param_option'
+    id = Column(Integer, primary_key=True)
+    validator_param = Column(Integer, ForeignKey('validator_param.id'))
+    option = Column(Integer)
+    force_permissive = Column(Boolean)
+    value = Column(PickleType)
+
+    def __init__(self, option=undefined, force_permissive=undefined,  value=undefined):
+        if value is not undefined:
+            self.value = value
+        elif option is not undefined:
+            self.option = option.id
+            self.force_permissive = force_permissive
+
+
+class _ValidatorParam(SqlAlchemyBase):
+    __tablename__ = 'validator_param'
+    id = Column(Integer, primary_key=True)
+    validator = Column(Integer, ForeignKey('baseoption.id'))
+    key = Column(String)
+    params = relationship('_ValidatorParamOption')
+
+    def __init__(self, key, params):
+        self.key = key
+        for param in params:
+            if isinstance(param, tuple):
+                if param == (None,):
+                    self.params.append(_ValidatorParamOption())
+                else:
+                    self.params.append(_ValidatorParamOption(option=param[0],
+                                                            force_permissive=param[1]))
+            else:
+                self.params.append(_ValidatorParamOption(value=param))
+
+
 #____________________________________________________________
 #
 # consistency
@@ -239,37 +388,36 @@ class _Base(SqlAlchemyBase):
     id = Column(Integer, primary_key=True)
     _name = Column(String)
     #FIXME not autoload
-    _infos = relationship("_Information",
-                          collection_class=attribute_mapped_collection('key'),
-                          cascade="all, delete-orphan")
-    _informations = association_proxy("_infos", "value")
+#    _infos = relationship("_Information",
+#                          collection_class=attribute_mapped_collection('key'),
+#                          cascade="all, delete-orphan")
+#    _informations = association_proxy("_infos", "value")
+    _informations = relationship("_Information")
     _default = Column(PickleType)
     _default_multi = Column(PickleType)
     _subdyn = Column(Integer)
     _dyn = Column(String)
     _opt = Column(Integer)
+    _master_slaves = Column(Integer)
     _choice_values = Column(PickleType)
-    _cho_params = relationship('_CallbackParam',
-                               collection_class=
-                               attribute_mapped_collection('key'))
-    _choice_params = association_proxy("_cho_params", "params",
-                                       getset_factory=load_callback_parm)
+    #_cho_params = relationship('_ChoiceParam',
+    #                           collection_class=attribute_mapped_collection('key'))
+    #_choice_values_params = association_proxy("_cho_params", "params",
+    #                                   getset_factory=load_choice_parm)
     _reqs = relationship("_Require", collection_class=list)
     _requires = association_proxy("_reqs", "requires", getset_factory=load_requires)
     _multi = Column(Integer)
     ######
     _callback = Column(PickleType)
     _call_params = relationship('_CallbackParam',
-                                collection_class=
-                                attribute_mapped_collection('key'))
+                                collection_class=attribute_mapped_collection('key'))
     _callback_params = association_proxy("_call_params", "params",
                                          getset_factory=load_callback_parm)
     _validator = Column(PickleType)
-    _val_params = relationship('_CallbackParam',
-                               collection_class=
-                               attribute_mapped_collection('key'))
+    _val_params = relationship('_ValidatorParam',
+                               collection_class=attribute_mapped_collection('key'))
     _validator_params = association_proxy("_val_params", "params",
-                                          getset_factory=load_callback_parm)
+                                          getset_factory=load_validator_parm)
     ######
     #FIXME not autoload
     _props = relationship("_PropertyOption", collection_class=set)
@@ -282,10 +430,10 @@ class _Base(SqlAlchemyBase):
     _consistencies = relationship('_Consistency', secondary=consistency_table,
                                   backref=backref('options',
                                                   enable_typechecks=False))
-    _type = Column(String(50))
     _stated = Column(Boolean)
+    _type = Column(String(50))
     __mapper_args__ = {
-        'polymorphic_identity': 'option',
+        'polymorphic_identity': 'optionsql',
         'polymorphic_on': _type
     }
     _extra = Column(PickleType)
@@ -293,10 +441,8 @@ class _Base(SqlAlchemyBase):
     _group_type = Column(String)
     _is_build_cache = Column(Boolean, default=False)
 
-    #def __init__(self):
     def __init__(self, name, multi, warnings_only, doc, extra, calc_properties,
-                 requires, properties, allow_empty_list, opt=undefined):
-        util.session.add(self)
+                 requires, properties, allow_empty_list, opt=undefined, session=None):
         self._name = name
         if multi is not undefined:
             self._multi = multi
@@ -305,7 +451,8 @@ class _Base(SqlAlchemyBase):
         if allow_empty_list is not undefined:
             self._allow_empty_list = allow_empty_list
         if doc is not undefined:
-            self._informations = {'doc': doc}
+            self._informations = [_Information(key='doc', value=doc)]
+            #self._informations = {'doc': doc}
         if opt is not undefined:
             self._opt = opt.id
         if extra is not undefined:
@@ -316,14 +463,19 @@ class _Base(SqlAlchemyBase):
             self._requires = requires
         if properties is not undefined:
             self._properties = properties
+        session.add(self)
 
-    def commit(self):
-        util.session.commit()
+    def getsession(self):
+        return util.Session()
+
+    def commit(self, session):
+        session.commit()
+        del(session)
 
     def _add_consistency(self, func, all_cons_opts, params):
         _Consistency(func, all_cons_opts, params)
 
-    def _set_default_values(self, default, default_multi):
+    def _set_default_values(self, default, default_multi, is_multi):
         self._default = default
         if self.impl_is_multi() and default_multi is not None:
             err = self._validate(default_multi)
@@ -339,10 +491,14 @@ class _Base(SqlAlchemyBase):
         return self.id
 
     def impl_get_callback(self):
+        a=self.getsession().query(_Base).filter_by(id=self.id).first()
         ret = self._callback
         if ret is None:
             return (None, {})
-        return ret, self._callback_params
+        params = self._callback_params
+        if params is None:
+            params = {}
+        return ret, params
 
     def impl_get_validator(self):
         ret = self._validator
@@ -351,16 +507,30 @@ class _Base(SqlAlchemyBase):
         return ret, self._validator_params
 
     def _impl_getsubdyn(self):
-        return util.session.query(_Base).filter_by(id=self._subdyn).first()
+        session = self.getsession()
+        return session.query(_Base).filter_by(id=self._subdyn).first()
 
     def _impl_getopt(self):
-        return util.session.query(_Base).filter_by(id=self._opt).first()
+        session = self.getsession()
+        return session.query(_Base).filter_by(id=self._opt).first()
 
     def impl_getname(self):
         return self._name
 
     def impl_getrequires(self):
-        return self._requires
+        session = self.getsession()
+        requires = session.query(_Require).filter_by(requires_id=self.id).all()
+        for require in requires:
+            _ret = []
+            for req in require.requires:
+                _ret.append((session.query(_Base).filter_by(id=req.option).first(),
+                       req.expected,
+                       req.action,
+                       req.inverse,
+                       req.transitive,
+                       req.same_action))
+            yield(_ret)
+            
 
     def impl_getdefault(self):
         ret = self._default
@@ -381,16 +551,60 @@ class _Base(SqlAlchemyBase):
         self._opt = opt.id
 
     def _impl_setsubdyn(self, subdyn):
+        session = self.getsession()
         self._subdyn = subdyn.id
-        self.commit()
+        self.commit(session)
 
-    def _set_readonly(self):
+    def _set_readonly(self, has_extra):
+        session = self.getsession()
+        opt = session.query(_Base).filter_by(id=self.id).first()
+        opt._readonly = True
+        session.commit()
         self._readonly = True
 
     def _set_callback(self, callback, callback_params):
         self._callback = callback
         if callback_params is not None:
-            self._callback_params = callback_params
+            opt._callback_params = callback_params
+        #session = self.getsession()
+        #opt = session.query(_Base).filter_by(id=self.id).first()
+        #opt._callback = callback
+        #if callback_params is not None:
+        #    opt._callback_params = callback_params
+        #session.commit()
+
+    def impl_set_choice_values_params(self, values, values_params, session):
+        self._choice_values = values
+        if values_params is not None:
+            for key, params in values_params.items():
+                choice = _ChoiceParam(self, key)
+                session.add(choice)
+                session.commit()
+                for param in params:
+                    if isinstance(param, tuple):
+                        if param == (None,):
+                            session.add(_ChoiceParamOption(choice))
+                        else:
+                            session.add(_ChoiceParamOption(choice, option=param[0], force_permissive=param[1]))
+                    else:
+                        session.add(_ChoiceParamOption(choice, value=param))
+        session.commit()
+
+    def impl_get_choice_values_params(self):
+        session = self.getsession()
+        params = {}
+        for param in session.query(_ChoiceParam).filter_by(option=self.id).all():
+            _params = []
+            for _param in session.query(_ChoiceParamOption).filter_by(choice=param.id).all():
+                if _param.value:
+                    _params.append(_param.value)
+                elif _param.option:
+                    _params.append((session.query(_Base).filter_by(id=_param.option).first(),
+                                         _param.force_permissive))
+                else:
+                    _params.append((None,))
+            params[param.key] = _params
+        return params
 
     def _set_validator(self, validator, validator_params):
         self._validator = validator
@@ -398,10 +612,11 @@ class _Base(SqlAlchemyBase):
             self._validator_params = validator_params
 
     def impl_is_readonly(self):
-        try:
-            return self._readonly
-        except AttributeError:
+        session = self.getsession()
+        opt = session.query(_Base).filter_by(id=self.id).first()
+        if opt is None or opt._readonly is None:
             return False
+        return opt._readonly
 
     def impl_is_multi(self):
         return self._multi == 0 or self._multi == 2
@@ -419,27 +634,38 @@ class _Base(SqlAlchemyBase):
         return self._warnings_only
 
     def impl_get_calc_properties(self):
-        try:
-            return self._calc_properties
-        except AttributeError:
-            return frozenset()
+        session = self.getsession()
+        return session.query(_CalcProperties).filter_by(require=self.id).all()
+        #try:
+        #    return self._calc_properties
+        #except AttributeError:
+        #    return frozenset()
 
     # information
     def impl_set_information(self, key, value):
-        self._informations[key] = value
+        session = self.getsession()
+        val = session.query(_Information).filter_by(
+            option=self.id, key=key).first()
+        if val is None:
+            session.add(_Information(self, key, value))
+        else:
+            val.value = value
+        session.commit()
 
     def impl_get_information(self, key, default=undefined):
         """retrieves one information's item
 
         :param key: the item string (ex: "help")
         """
-        if default is not undefined:
-            return self._informations.get(key, default)
-        try:
-            return self._informations[key]
-        except KeyError:  # pragma: optional cover
+        session = self.getsession()
+        val = session.query(_Information).filter_by(
+            option=self.id, key=key).first()
+        if not val:
+            if default is not undefined:
+                return default
             raise ValueError(_("information's item not found: {0}").format(
-                key))
+                               key))
+        return val.value
 
     def _impl_getattributes(self):
         slots = set()
@@ -448,6 +674,21 @@ class _Base(SqlAlchemyBase):
                 slots.add(column.key)
         return slots
 
+    def impl_getproperties(self):
+        session = self.getsession()
+        for prop in session.query(_PropertyOption).filter_by(option=self.id).all():
+            yield prop.name
+
+    def _set_master_slaves(self, option):
+        session = self.getsession()
+        opt = session.query(_Base).filter_by(id=self.id).first()
+        opt._master_slaves = option._p_.id
+        self.commit(session)
+
+    def _get_master_slave(self):
+        session = self.getsession()
+        return session.query(StorageMasterSlaves).filter_by(id=self._master_slaves).first()
+
 
 class Cache(SqlAlchemyBase):
     __tablename__ = 'cache'
@@ -477,19 +718,24 @@ class Cache(SqlAlchemyBase):
 
 class StorageOptionDescription(object):
     def impl_already_build_caches(self):
-        return self._is_build_cache
+        cache = self._is_build_cache
+        if cache is None:
+            cache = False
+        return cache
 
     def impl_get_opt_by_path(self, path):
-        ret = util.session.query(Cache).filter_by(descr=self.id, path=path).first()
+        session = self.getsession()
+        ret = session.query(Cache).filter_by(descr=self.id, path=path).first()
         if ret is None:
             raise AttributeError(_('no option for path {0}').format(path))
-        return util.session.query(_Base).filter_by(id=ret.option).first()
+        return session.query(_Base).filter_by(id=ret.option).first()
 
     def impl_get_path_by_opt(self, opt):
-        ret = util.session.query(Cache).filter_by(descr=self.id,
+        session = self.getsession()
+        ret = session.query(Cache).filter_by(descr=self.id,
                                                   option=opt.id).first()
         if ret is None:
-            ret = util.session.query(Cache).filter_by(descr=self.id).first()
+            ret = session.query(Cache).filter_by(descr=self.id).first()
             if ret is None:
                 raise ConfigError(_('use impl_get_path_by_opt only with root OptionDescription'))
             raise AttributeError(_('no option {0} found').format(opt))
@@ -499,11 +745,15 @@ class StorageOptionDescription(object):
         return getattr(groups, self._group_type)
 
     def impl_build_cache_option(self, descr=None, _currpath=None,
-                                subdyn_path=None):
+                                subdyn_path=None, session=None):
+        if self.impl_is_readonly() or (_currpath is None and getattr(self, '_cache_paths', None) is not None):
+            # cache already set
+            return
         if descr is None:
             save = True
             descr = self
             _currpath = []
+            session = self.getsession()
         else:
             save = False
         for option in self._impl_getchildren(dyn=False):
@@ -512,23 +762,23 @@ class StorageOptionDescription(object):
                 sub = subdyn_path
                 if option.impl_is_dynoptiondescription():
                     sub = '.'.join(_currpath)
-                util.session.add(Cache(descr, self, option,
+                session.add(Cache(descr, self, option,
                                        str('.'.join(_currpath + [attr])),
                                        sub))
                 _currpath.append(attr)
                 option.impl_build_cache_option(descr,
                                                _currpath,
-                                               sub)
+                                               sub, session)
                 _currpath.pop()
             else:
                 if subdyn_path:
                     subdyn_path = '.'.join(_currpath)
-                util.session.add(Cache(descr, self, option,
+                session.add(Cache(descr, self, option,
                                        str('.'.join(_currpath + [attr])),
                                        subdyn_path))
         if save:
             self._is_build_cache = True
-            util.session.commit()
+            self.commit(session)
 
     def impl_get_options_paths(self, bytype, byname, _subpath, only_first,
                                context):
@@ -546,7 +796,8 @@ class StorageOptionDescription(object):
             _opt = option._impl_to_dyn(name + suffix, path)
             return (path, _opt)
 
-        sqlquery = util.session.query(Cache).filter_by(descr=self.id)
+        session = self.getsession()
+        sqlquery = session.query(Cache).filter_by(descr=self.id)
         if bytype is None:
             sqlquery = sqlquery.filter(and_(not_(
                 Cache.opt_type == 'OptionDescription'),
@@ -575,7 +826,7 @@ class StorageOptionDescription(object):
         #else:
         ret = []
         for opt in sqlquery.all():
-            option = util.session.query(_Base).filter_by(id=opt.option).first()
+            option = session.query(_Base).filter_by(id=opt.option).first()
             if opt.is_subdyn:
                 name = option.impl_getname()
                 if byname is not None:
@@ -616,21 +867,24 @@ class StorageOptionDescription(object):
         return ret
 
     def _add_children(self, child_names, children):
+        session = self.getsession()
         for child in children:
-            util.session.add(_Parent(self, child))
+            session.add(_Parent(self, child))
+        self.commit(session)
 
     def _impl_st_getchildren(self, context, only_dyn=False):
+        session = self.getsession()
         if only_dyn is False or context is undefined:
-            for child in util.session.query(_Parent).filter_by(
+            for child in session.query(_Parent).filter_by(
                     parent_id=self.id).all():
-                yield(util.session.query(_Base).filter_by(id=child.child_id
+                yield(session.query(_Base).filter_by(id=child.child_id
                                                           ).first())
         else:
             descr = context.cfgimpl_get_description().id
-            for child in util.session.query(Cache).filter_by(descr=descr,
+            for child in session.query(Cache).filter_by(descr=descr,
                                                              parent=self.id
                                                              ).all():
-                yield(util.session.query(_Base).filter_by(id=child.option).first())
+                yield(session.query(_Base).filter_by(id=child.option).first())
 
     def _getattr(self, name, suffix=undefined, context=undefined, dyn=True):
         error = False
@@ -640,14 +894,15 @@ class StorageOptionDescription(object):
                     raise ConfigError(_("suffix and context needed if "
                                         "it's a dyn option"))
                 if name.endswith(suffix):
+                    session = self.getsession()
                     oname = name[:-len(suffix)]
                     #child = self._children[1][self._children[0].index(oname)]
-                    child = util.session.query(_Parent).filter_by(
+                    child = session.query(_Parent).filter_by(
                         parent_id=self.id, child_name=oname).first()
                     if child is None:
                         error = True
                     else:
-                        opt = util.session.query(_Base).filter_by(
+                        opt = session.query(_Base).filter_by(
                             id=child.child_id).first()
                         return self._impl_get_dynchild(opt, suffix)
                 else:
@@ -655,7 +910,8 @@ class StorageOptionDescription(object):
             except ValueError:  # pragma: optional cover
                 error = True
         else:
-            child = util.session.query(_Parent).filter_by(parent_id=self.id,
+            session = self.getsession()
+            child = session.query(_Parent).filter_by(parent_id=self.id,
                                                           child_name=name
                                                           ).first()
             if child is None:
@@ -664,7 +920,7 @@ class StorageOptionDescription(object):
                     return child
                 error = True
             if error is False:
-                return util.session.query(_Base).filter_by(id=child.child_id
+                return session.query(_Base).filter_by(id=child.child_id
                                                            ).first()
         if error:
             raise AttributeError(_('unknown Option {0} in OptionDescription {1}'
@@ -672,12 +928,13 @@ class StorageOptionDescription(object):
 
     def _get_force_store_value(self):
         #only option in current tree
-        current_ids = tuple(chain(*util.session.query(Cache.option).filter_by(
+        session = self.getsession()
+        current_ids = tuple(chain(*session.query(Cache.option).filter_by(
             descr=self.id).all()))
-        for prop in util.session.query(_PropertyOption).filter(
+        for prop in session.query(_PropertyOption).filter(
                 _PropertyOption.option.in_(current_ids),
                 _PropertyOption.name == 'force_store_value').all():
-            opt = util.session.query(_Base).filter_by(id=prop.option).first()
+            opt = session.query(_Base).filter_by(id=prop.option).first()
             path = self.impl_get_path_by_opt(opt)
             yield (opt, path)
 
@@ -686,3 +943,39 @@ class StorageBase(_Base):
     @declared_attr
     def __mapper_args__(self):
         return {'polymorphic_identity': self.__name__.lower()}
+
+
+class _Slave(SqlAlchemyBase):
+    __tablename__ = 'slaves'
+    id = Column(Integer, primary_key=True)
+    master_id = Column(Integer, index=True, nullable=False)
+    slave_id = Column(Integer)
+
+    def __init__(self, master, slave):
+        self.master_id = master.id
+        self.slave_id = slave.id
+
+
+class StorageMasterSlaves(SqlAlchemyBase):
+    __tablename__ = 'masterslaves2'
+    id = Column(Integer, primary_key=True)
+    master = Column(Integer)
+
+    def __init__(self, master, slaves):
+        session = util.Session()
+        self.master = master.id
+        session.add(self)
+        session.commit()
+        for slave in slaves:
+            sl = _Slave(self, slave)
+            session.add(sl)
+        session.commit()
+
+    def _sm_getslaves(self):
+        session = util.Session()
+        for slave in session.query(_Slave).filter_by(master_id=self.master).all():
+            yield(session.query(_Base).filter_by(id=slave.slave_id).first())
+
+    def _sm_getmaster(self):
+        session = util.Session()
+        return session.query(_Base).filter_by(id=self.master).first()
