@@ -19,7 +19,7 @@ from ..setting import owners
 
 
 class SerializeObject(object):
-    def __getstate__(self):
+    def __getstate__(self):  # pragma: no cover
         ret = {}
         for key in dir(self):
             if not key.startswith('__'):
@@ -53,7 +53,7 @@ class Cache(object):
                     str_owner = []
                     _value.append(value[2])
                     for owner in value[3]:
-                        if isinstance(owner, list):
+                        if isinstance(owner, list):  # pragma: no cover
                             str_owners = []
                             for subowner in owner:
                                 str_owners.append(str(subowner))
@@ -64,7 +64,7 @@ class Cache(object):
                     states[slot] = _value
                 else:
                     states[slot] = value
-            except AttributeError:
+            except AttributeError:  # pragma: no cover
                 pass
         return states
 
@@ -72,7 +72,7 @@ class Cache(object):
         def convert_owner(owner):
             try:
                 owner = getattr(owners, owner)
-            except AttributeError:
+            except AttributeError:  # pragma: no cover
                 owners.addowner(owner)
                 owner = getattr(owners, owner)
             return owner
@@ -86,7 +86,7 @@ class Cache(object):
                 _value.append(value[2])
                 obj_owner = []
                 for owner in value[3]:
-                    if isinstance(owner, list):
+                    if isinstance(owner, list):  # pragma: no cover
                         obj_owners = []
                         for subowner in owner:
                             obj_owners.append(convert_owner(subowner))
@@ -104,7 +104,7 @@ class Cache(object):
         value, created = self._cache[path][index]
         if created is None or exp <= created:
             return True, value
-        return False, None
+        return False, None  # pragma: no cover
 
     def hascache(self, path, index):
         """ path is in the cache
