@@ -50,29 +50,29 @@ class Values(object):
         old `SubConfig`, `Values`, `Multi` or `Settings`)
         """
         context = self.context()
-        if context is None:  # pragma: optional cover
+        if context is None:
             raise ConfigError(_('the context does not exist anymore'))
         return context
 
     def _get_multi(self, opt, path):
         return Multi([], self.context, opt, path)
 
-    def _getdefaultvalue(self, opt, path, with_meta, index, submulti_index,
-                         returns_raise):
+    def _getdefaultvalue(self, opt, path, with_meta, index, submulti_index, validate):
         # if value has callback and is not set
         if opt.impl_has_callback():
             callback, callback_params = opt.impl_get_callback()
             value = carry_out_calculation(opt, context=self._getcontext(),
                                           callback=callback,
                                           callback_params=callback_params,
-                                          index=index,
-                                          returns_raise=returns_raise)
+                                          index=index, validate=validate)
             if isinstance(value, list) and index is not None:
                 #if return a list and index is set, return value only if
                 #it's a submulti without submulti_index and without list of list
                 if opt.impl_is_submulti() and submulti_index is undefined and \
                         (len(value) == 0 or not isinstance(value[0], list)):
                     return value
+                if not opt.impl_is_submulti() and len(value) > index:
+                    return value[index]
             else:
                 return value
         if with_meta:
@@ -80,16 +80,27 @@ class Values(object):
             if meta is not None:
                 value = meta.cfgimpl_get_values(
                 )._get_cached_value(opt, path, index=index, submulti_index=submulti_index,
-                                    from_masterslave=True, returns_raise=True)
+                                    from_masterslave=True)
                 if isinstance(value, Exception):
-                    if not isinstance(value, PropertiesOptionError):
+                    if not isinstance(value, PropertiesOptionError):  # pragma: no cover
                         raise value
                 else:
                     if isinstance(value, Multi):
                         if index is not None:
                             value = value[index]
+                            if isinstance(value, SubMulti):
+                                if submulti_index is not undefined:
+                                    value = value[submulti_index]
+                                else:
+                                    value = list(value)
                         else:
-                            value = list(value)
+                            new_value = []
+                            for val in value:
+                                if isinstance(val, SubMulti):
+                                    val = list(val)
+                                new_value.append(val)
+                            value = new_value
+                            del new_value
                     return value
         # now try to get default value
         value = opt.impl_getdefault()
@@ -104,7 +115,7 @@ class Values(object):
         return value
 
     def _getvalue(self, opt, path, self_properties, index, submulti_index,
-                  with_meta, masterlen, session):
+                  with_meta, masterlen, session, validate):
         """actually retrieves the value
 
         :param opt: the `option.Option()` object
@@ -113,11 +124,11 @@ class Values(object):
         force_default = 'frozen' in self_properties and \
             'force_default_on_freeze' in self_properties
         # not default value
-        is_default = self._is_default_owner(opt, path, session,
-                                            validate_properties=False,
-                                            validate_meta=False,
-                                            self_properties=self_properties,
-                                            index=index)
+        if index is None or not opt.impl_is_master_slaves('slave'):
+            _index = None
+        else:
+            _index = index
+        is_default = self._p_.getowner(path, owners.default, session, only_default=True, index=_index) == owners.default
         if not is_default and not force_default:
             if opt.impl_is_master_slaves('slave'):
                 return self._p_.getvalue(path, session, index)
@@ -131,7 +142,7 @@ class Values(object):
                 else:
                     return value
         return self._getdefaultvalue(opt, path, with_meta, index,
-                                     submulti_index, True)
+                                     submulti_index, validate)
 
     def get_modified_values(self):
         return self._p_.get_modified_values()
@@ -170,9 +181,11 @@ class Values(object):
             fake_context = context._gen_fake_values(session)
             fake_value = fake_context.cfgimpl_get_values()
             fake_value.reset(opt, path, validate=False)
-            fake_value._get_cached_value(opt, path,
-                                         setting_properties=_setting_properties,
-                                         check_frozen=True)
+            ret = fake_value._get_cached_value(opt, path,
+                                               setting_properties=_setting_properties,
+                                               check_frozen=True)
+            if isinstance(ret, Exception):
+                raise ret
         if opt.impl_is_master_slaves('master'):
             opt.impl_get_master_slaves().reset(opt, self, _setting_properties)
         if hasvalue:
@@ -181,7 +194,9 @@ class Values(object):
                                                              setting_properties=_setting_properties,
                                                              read_write=False,
                                                              apply_requires=False):
-                value = self._getdefaultvalue(opt, path, True, undefined, undefined, False)
+                value = self._getdefaultvalue(opt, path, True, undefined, undefined, validate)
+                if isinstance(value, Exception):  # pragma: no cover
+                    raise value
                 self._setvalue(opt, path, value, force_owner=owners.forced)
             else:
                 self._p_.resetvalue(path, session)
@@ -203,7 +218,7 @@ class Values(object):
                             allow_empty_list = True
                         else:
                             allow_empty_list = False
-                isempty = (not allow_empty_list and value == []) or \
+                isempty = value is None or (not allow_empty_list and value == []) or \
                     None in value or empty in value
             else:
                 isempty = value is None or value == empty
@@ -211,7 +226,7 @@ class Values(object):
 
     def __getitem__(self, opt):
         "enables us to use the pythonic dictionary-like access to values"
-        return self.getitem(opt)
+        return self._get_cached_value(opt)
 
     def getitem(self, opt, validate=True, force_permissive=False):
         """
@@ -225,7 +240,7 @@ class Values(object):
                           setting_properties=undefined, self_properties=undefined,
                           index=None, submulti_index=undefined, from_masterslave=False,
                           with_meta=True, masterlen=undefined, check_frozen=False,
-                          returns_raise=False, session=None):
+                          session=None, display_warnings=True):
         context = self._getcontext()
         settings = context.cfgimpl_get_settings()
         if path is None:
@@ -254,10 +269,7 @@ class Values(object):
                                                          self_properties=self_properties,
                                                          index=index)
                     if props:
-                        if returns_raise:
-                            return props
-                        else:
-                            raise props
+                        return props
                 return value
         if session is None:
             session = self._p_.getsession()
@@ -271,25 +283,22 @@ class Values(object):
                                                        setting_properties=setting_properties,
                                                        index=index,
                                                        self_properties=self_properties,
-                                                       returns_raise=returns_raise)
+                                                       check_frozen=check_frozen)
         else:
             val = self._get_validated_value(opt, path, validate,
                                             force_permissive,
                                             validate_properties,
-                                            setting_properties=setting_properties,
-                                            self_properties=self_properties,
+                                            setting_properties,
+                                            self_properties,
                                             with_meta=with_meta,
                                             masterlen=masterlen,
                                             index=index,
                                             submulti_index=submulti_index,
                                             check_frozen=check_frozen,
-                                            returns_raise=returns_raise,
-                                            session=session)
+                                            session=session,
+                                            display_warnings=display_warnings)
         if isinstance(val, Exception):
-            if returns_raise:
-                return val
-            else:
-                raise val
+            return val
         # cache doesn't work with SubMulti yet
         if not isinstance(val, SubMulti) and 'cache' in setting_properties and \
                 validate and validate_properties and force_permissive is False \
@@ -302,27 +311,25 @@ class Values(object):
         return val
 
     def _get_validated_value(self, opt, path, validate, force_permissive,
-                             validate_properties,
+                             validate_properties, setting_properties,
+                             self_properties,
                              index=None, submulti_index=undefined,
-                             with_meta=True, setting_properties=undefined,
-                             self_properties=undefined, masterlen=undefined,
-                             check_frozen=False, returns_raise=False,
-                             session=None):
+                             with_meta=True,
+                             masterlen=undefined,
+                             check_frozen=False,
+                             session=None, display_warnings=True):
         """same has getitem but don't touch the cache
         index is None for slave value, if value returned is not a list, just return []
         """
         context = self._getcontext()
         setting = context.cfgimpl_get_settings()
-        if setting_properties is undefined:
-            setting_properties = setting._getproperties(read_write=False)
-        if self_properties is undefined:
-            self_properties = setting._getproperties(opt, path, read_write=False, index=index)
         config_error = None
         if session is None:
             session = self._p_.getsession()
         value = self._getvalue(opt, path, self_properties, index, submulti_index,
-                               with_meta, masterlen, session)
+                               with_meta, masterlen, session, validate)
         if isinstance(value, Exception):
+            value_error = True
             if isinstance(value, ConfigError):
                 # For calculating properties, we need value (ie for mandatory
                 # value).
@@ -335,19 +342,16 @@ class Values(object):
                 # value is not set, for 'undefined' (cannot set None because of
                 # mandatory property)
                 value = undefined
-            else:
+            else:  # pragma: no cover
                 raise value
         else:
-            if index is undefined:
-                force_index = None
-            else:
-                force_index = index
+            value_error = False
             if opt.impl_is_multi():
-                if force_index is None:
+                if index is None:
                     value = Multi(value, self.context, opt, path)
                 elif opt.impl_is_submulti() and submulti_index is undefined:
                     value = SubMulti(value, self.context, opt, path,
-                                     force_index)
+                                     index)
 
             if validate:
                 if submulti_index is undefined:
@@ -356,8 +360,10 @@ class Values(object):
                     force_submulti_index = submulti_index
                 err = opt.impl_validate(value, context,
                                         'validator' in setting_properties,
-                                        force_index=force_index,
-                                        force_submulti_index=force_submulti_index)
+                                        force_index=index,
+                                        force_submulti_index=force_submulti_index,
+                                        display_error=True,
+                                        display_warnings=False)
                 if err:
                     config_error = err
                     value = None
@@ -376,22 +382,23 @@ class Values(object):
                                                 self_properties=self_properties,
                                                 index=index)
             if props:
-                if returns_raise:
-                    return props
-                else:
-                    raise props
+                return props
+        if not value_error and validate and display_warnings:
+            opt.impl_validate(value, context,
+                              'validator' in setting_properties,
+                              force_index=index,
+                              force_submulti_index=force_submulti_index,
+                              display_error=False,
+                              display_warnings=display_warnings)
         if config_error is not None:
-            if returns_raise:
-                return config_error
-            else:
-                raise config_error
+            return config_error
         return value
 
-    def __setitem__(self, opt, value):  # pragma: optional cover
+    def __setitem__(self, opt, value):
         raise ConfigError(_('you should only set value with config'))
 
     def setitem(self, opt, value, path, force_permissive=False,
-                check_frozen=True, not_raises=False):
+                check_frozen=True, not_raises=False, index=None):
         # check_frozen is, for example, used with "force_store_value"
         # user didn't change value, so not write
         # valid opt
@@ -401,18 +408,20 @@ class Values(object):
             session = context.cfgimpl_get_values()._p_.getsession()
             fake_context = context._gen_fake_values(session)
             fake_values = fake_context.cfgimpl_get_values()
-            fake_values._setvalue(opt, path, value)
+            fake_values._setvalue(opt, path, value, index=index)
             props = fake_values.validate(opt, value, path,
                                          check_frozen=check_frozen,
                                          force_permissive=force_permissive,
                                          setting_properties=setting_properties,
-                                         session=session, not_raises=not_raises)
+                                         session=session, not_raises=not_raises,
+                                         index=index)
             if props and not_raises:
                 return
-            err = opt.impl_validate(value, fake_context)
+            err = opt.impl_validate(value, fake_context, display_warnings=False, force_index=index)
             if err:
                 raise err
-        self._setvalue(opt, path, value)
+            opt.impl_validate(value, fake_context, display_error=False)
+        self._setvalue(opt, path, value, index=index)
 
     def _setvalue(self, opt, path, value, force_owner=undefined, index=None):
         context = self._getcontext()
@@ -423,16 +432,19 @@ class Values(object):
             owner = force_owner
         # in storage, value must not be a multi
         if isinstance(value, Multi):
-            value = list(value)
-            if opt.impl_is_submulti():
-                for idx, val in enumerate(value):
-                    if isinstance(val, SubMulti):
-                        value[idx] = list(val)
+            if not opt.impl_is_master_slaves('slave') or index is None:
+                value = list(value)
+                if opt.impl_is_submulti():
+                    for idx, val in enumerate(value):
+                        if isinstance(val, SubMulti):
+                            value[idx] = list(val)
+            else:
+                value = value[index]
         session = self._p_.getsession()
         #FIXME pourquoi là et pas dans masterslaves ??
         if opt.impl_is_master_slaves('slave'):
             if index is not None:
-                self._p_.setvalue(path, value[index], owner, index, session)
+                self._p_.setvalue(path, value, owner, index, session)
             else:
                 self._p_.resetvalue(path, session)
                 for idx, val in enumerate(value):
@@ -443,18 +455,27 @@ class Values(object):
 
     def validate(self, opt, value, path, check_frozen=True, force_permissive=False,
                  setting_properties=undefined, valid_masterslave=True,
-                 not_raises=False, returns_raise=False, session=None):
+                 not_raises=False, session=None, index=None):
         if valid_masterslave and opt.impl_is_master_slaves():
             if session is None:
                 session = self._p_.getsession()
-            opt.impl_get_master_slaves().validate(self, opt, value, path, returns_raise, session)
+            if index is not None:
+                len_value = index
+                setitem = False
+            else:
+                len_value = len(value)
+                setitem = True
+            val = opt.impl_get_master_slaves().validate(self, opt, len_value, path, session, setitem=setitem)
+            if isinstance(val, Exception):
+                return val
         props = self._getcontext().cfgimpl_get_settings().validate_properties(opt,
                                                                               False,
                                                                               check_frozen,
                                                                               value=value,
                                                                               path=path,
                                                                               force_permissive=force_permissive,
-                                                                              setting_properties=setting_properties)
+                                                                              setting_properties=setting_properties,
+                                                                              index=index)
         if props:
             if not_raises:
                 return props
@@ -505,8 +526,12 @@ class Values(object):
         if 'frozen' in self_properties and 'force_default_on_freeze' in self_properties:
             return owners.default
         if validate_properties:
-            self._get_cached_value(opt, path, True, force_permissive, None, True,
-                                   self_properties=self_properties, session=session)
+            value = self._get_cached_value(opt, path, True, force_permissive, None, True,
+                                           self_properties=self_properties, session=session,
+                                           index=index)
+            if isinstance(value, Exception):
+                raise value
+
         owner = self._p_.getowner(path, owners.default, session, only_default=only_default, index=index)
         if validate_meta is undefined:
             if opt.impl_is_master_slaves('slave'):
@@ -532,12 +557,12 @@ class Values(object):
         :param opt: the `option.Option` object
         :param owner: a valid owner, that is a `setting.owners.Owner` object
         """
-        if not isinstance(owner, owners.Owner):  # pragma: optional cover
+        if not isinstance(owner, owners.Owner):
             raise TypeError(_("invalid generic owner {0}").format(str(owner)))
 
         path = opt.impl_getpath(self._getcontext())
         session = self._p_.getsession()
-        if not self._p_.hasvalue(path, session):  # pragma: optional cover
+        if not self._p_.hasvalue(path, session):
             raise ConfigError(_('no value for {0} cannot change owner to {1}'
                                 '').format(path, owner))
         props = self._getcontext().cfgimpl_get_settings().validate_properties(opt,
@@ -566,8 +591,6 @@ class Values(object):
     def _is_default_owner(self, opt, path, session, validate_properties=True,
                           validate_meta=True, self_properties=undefined,
                           index=None, force_permissive=False):
-        if not opt.impl_is_master_slaves('slave'):
-            index = None
         d = self._getowner(opt, path, session, validate_properties=validate_properties,
                            validate_meta=validate_meta,
                            self_properties=self_properties, only_default=True,
@@ -602,21 +625,27 @@ class Values(object):
     def del_information(self, key, raises=True):
         self._p_.del_information(key, raises)
 
-    def mandatory_warnings(self, force_permissive=False, validate=True):
+    def mandatory_warnings(self, force_permissive=True):
         """convenience function to trace Options that are mandatory and
         where no value has been set
 
-        :param force_permissive: do raise with permissives properties
-        :type force_permissive: `bool`
-        :param validate: validate value when calculating properties
-        :type validate: `bool`
-
         :returns: generator of mandatory Option's path
         """
         context = self._getcontext()
         settings = context.cfgimpl_get_settings()
         setting_properties = context.cfgimpl_get_settings()._getproperties()
         setting_properties.update(['mandatory', 'empty'])
+        def _is_properties_option(err, path):
+            if not isinstance(err, Exception):
+                pass
+            elif isinstance(err, PropertiesOptionError):
+                if err.proptype == ['mandatory']:
+                    return path
+            elif isinstance(err, ConfigError):
+                #assume that uncalculated value is an empty value
+                return path
+            else:
+                raise err
 
         def _mandatory_warnings(description, currpath=None):
             if currpath is None:
@@ -627,7 +656,7 @@ class Values(object):
 
                 if opt.impl_is_optiondescription():
                     if not settings.validate_properties(opt, True, False, path=path,
-                                                        force_permissive=force_permissive,
+                                                        force_permissive=True,
                                                         setting_properties=setting_properties):
                         for path in _mandatory_warnings(opt, currpath + [name]):
                             yield path
@@ -641,23 +670,21 @@ class Values(object):
                     if 'mandatory' in self_properties or 'empty' in self_properties:
                         err = self._get_cached_value(opt, path=path,
                                                      trusted_cached_properties=False,
-                                                     force_permissive=force_permissive,
+                                                     force_permissive=True,
                                                      setting_properties=setting_properties,
                                                      self_properties=self_properties,
-                                                     validate=validate, returns_raise=True)
-                        if not isinstance(err, Exception):
-                            pass
-                        elif isinstance(err, PropertiesOptionError):
-                            if err.proptype == ['mandatory']:
-                                yield path
-                        elif isinstance(err, ConfigError):
-                            if validate:
-                                raise err
-                            else:
-                                #assume that uncalculated value is an empty value
-                                yield path
+                                                     validate=True,
+                                                     display_warnings=False)
+                        if opt.impl_is_master_slaves('slave') and isinstance(err, list):
+                            for val in err:
+                                ret = _is_properties_option(val, path)
+                                if ret is not None:
+                                    yield ret
+                                    break
                         else:
-                            raise err
+                            ret = _is_properties_option(err, path)
+                            if ret is not None:
+                                yield ret
 
         descr = self._getcontext().cfgimpl_get_description()
         for path in _mandatory_warnings(descr):
@@ -675,7 +702,7 @@ class Values(object):
         for path in context.cfgimpl_get_description().impl_getpaths(
                 include_groups=True):
             err = context.getattr(path, returns_raise=True)
-            if isinstance(err, Exception) and not isinstance(err, PropertiesOptionError):
+            if isinstance(err, Exception) and not isinstance(err, PropertiesOptionError):  # pragma: no cover
                 raise err
 
     def __getstate__(self):
@@ -704,12 +731,12 @@ class Multi(list):
         """
         if value is None:
             value = []
-        if not opt.impl_is_submulti() and isinstance(value, Multi):  # pragma: optional cover
+        if not opt.impl_is_submulti() and isinstance(value, Multi):
             raise ValueError(_('{0} is already a Multi ').format(
                 opt.impl_getname()))
         self.opt = opt
         self.path = path
-        if not isinstance(context, weakref.ReferenceType):  # pragma: optional cover
+        if not isinstance(context, weakref.ReferenceType):
             raise ValueError('context must be a Weakref')
         self.context = context
         if not isinstance(value, list):
@@ -738,7 +765,7 @@ class Multi(list):
         old `SubConfig`, `Values`, `Multi` or `Settings`)
         """
         context = self.context()
-        if context is None:  # pragma: optional cover
+        if context is None:
             raise ConfigError(_('the context does not exist anymore'))
         return context
 
@@ -754,8 +781,7 @@ class Multi(list):
         if 'validator' in setting_properties and validate:
             session = context.cfgimpl_get_values()._p_.getsession()
             fake_context = context._gen_fake_values(session)
-            fake_multi = fake_context.cfgimpl_get_values()._get_cached_value(
-                self.opt, path=self.path, validate=False)
+            fake_multi = Multi(list(self), weakref.ref(fake_context), self.opt, self.path)
             fake_multi._setitem(index, value, validate=False)
             self._validate(value, fake_context, index, True)
         #assume not checking mandatory property
@@ -771,23 +797,28 @@ class Multi(list):
             raise value
         return value
 
-    def _get_validated_value(self, index):
+    def __delitem__(self, index):
+        return self.pop(index)
+
+    def _getdefaultvalue(self, index):
         values = self._getcontext().cfgimpl_get_values()
-        return values._get_validated_value(self.opt, self.path,
-                                           True, False, True,
-                                           index=index)
+        value = values._getdefaultvalue(self.opt, self.path, True, index,
+                                        undefined, True)
+        if self.opt.impl_is_submulti():
+            value = SubMulti(value, self.context, self.opt, self.path, index)
+        return value
 
     def append(self, value=undefined, force=False, setitem=True, validate=True,
                force_permissive=False):
         """the list value can be updated (appened)
         only if the option is a master
         """
-        if not force and self.opt.impl_is_master_slaves('slave'):  # pragma: optional cover
+        if not force and self.opt.impl_is_master_slaves('slave'):
             raise SlaveError(_("cannot append a value on a multi option {0}"
                                " which is a slave").format(self.opt.impl_getname()))
         index = self.__len__()
         if value is undefined:
-            value = self._get_validated_value(index)
+            value = self._getdefaultvalue(index)
         if validate and value not in [None, undefined]:
             context = self._getcontext()
             setting = context.cfgimpl_get_settings()
@@ -795,9 +826,9 @@ class Multi(list):
             if 'validator' in setting_properties:
                 session = context.cfgimpl_get_values()._p_.getsession()
                 fake_context = context._gen_fake_values(session)
-                fake_multi = fake_context.cfgimpl_get_values()._get_cached_value(
-                    self.opt, path=self.path, validate=False,
-                    force_permissive=force_permissive)
+                fake_multi = Multi(list(self), weakref.ref(fake_context), self.opt, self.path)
+                if isinstance(fake_multi, Exception):
+                    raise fake_multi
                 fake_multi.append(value, validate=False, force=True,
                                   setitem=setitem)
                 self._validate(value, fake_context, index, True)
@@ -816,7 +847,7 @@ class Multi(list):
         if self.opt.impl_is_master_slaves():
             raise SlaveError(_("cannot sort multi option {0} if master or slave"
                                "").format(self.opt.impl_getname()))
-        if sys.version_info[0] >= 3:
+        if sys.version_info[0] >= 3:  # pragma: no cover
             if cmp is not None:
                 raise ValueError(_('cmp is not permitted in python v3 or '
                                    'greater'))
@@ -842,8 +873,7 @@ class Multi(list):
         if 'validator' in setting_properties and validate and value is not None:
             session = context.cfgimpl_get_values()._p_.getsession()
             fake_context = context._gen_fake_values(session)
-            fake_multi = fake_context.cfgimpl_get_values()._get_cached_value(
-                self.opt, path=self.path, validate=False)
+            fake_multi = Multi(list(self), weakref.ref(fake_context), self.opt, self.path)
             fake_multi.insert(index, value, validate=False)
             self._validate(value, fake_context, index, True)
         super(Multi, self).insert(index, value)
@@ -860,16 +890,20 @@ class Multi(list):
         if 'validator' in setting_properties and validate:
             session = context.cfgimpl_get_values()._p_.getsession()
             fake_context = context._gen_fake_values(session)
-            fake_multi = fake_context.cfgimpl_get_values()._get_cached_value(
-                self.opt, path=self.path, validate=False)
-            fake_multi.extend(iterable, validate=False)
-            self._validate(iterable, fake_context, index)
+            fake_multi = Multi(list(self), weakref.ref(fake_context), self.opt, self.path)
+            if index is None:
+                fake_multi.extend(iterable, validate=False)
+                self._validate(fake_multi, fake_context, index)
+            else:
+                fake_multi[index].extend(iterable, validate=False)
+                self._validate(fake_multi[index], fake_context, index)
         super(Multi, self).extend(iterable)
         self._store()
 
     def _validate(self, value, fake_context, force_index, submulti=False):
         err = self.opt.impl_validate(value, context=fake_context,
-                                     force_index=force_index)
+                                     force_index=force_index,
+                                     multi=self)
         if err:
             raise err
 
@@ -885,7 +919,7 @@ class Multi(list):
         """
         context = self._getcontext()
         if not force:
-            if self.opt.impl_is_master_slaves('slave'):  # pragma: optional cover
+            if self.opt.impl_is_master_slaves('slave'):
                 raise SlaveError(_("cannot pop a value on a multi option {0}"
                                    " which is a slave").format(self.opt.impl_getname()))
             if self.opt.impl_is_master_slaves('master'):
@@ -896,6 +930,10 @@ class Multi(list):
         self._store(force=force)
         return ret
 
+    def remove(self, value):
+        idx = self.index(value)
+        return self.pop(idx)
+
     def _store(self, force=False, index=None):
         values = self._getcontext().cfgimpl_get_values()
         if not force:
@@ -939,13 +977,12 @@ class SubMulti(Multi):
             else:
                 err = self.opt.impl_validate(value, context=fake_context,
                                              force_index=self._index,
-                                             force_submulti_index=force_index)
+                                             force_submulti_index=force_index,
+                                             multi=self)
                 if err:
                     raise err
 
-    def _get_validated_value(self, index):
+    def _getdefaultvalue(self, index):
         values = self._getcontext().cfgimpl_get_values()
-        return values._get_validated_value(self.opt, self.path,
-                                           True, False, True,
-                                           index=index,
-                                           submulti_index=self._index)
+        return values._getdefaultvalue(self.opt, self.path, True, index,
+                                       self._index, True)
